@@ -7,7 +7,7 @@ from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
 
 def as_signed(val: int):
     # converts an integer to a signed int
-    return struct.unpack('<i', struct.pack('<I', val & 0xffffffff))[0]
+    return struct.unpack('<i', struct.pack('<I', val & 0xFFFFFFFF))[0]
 
 
 def aes_cbc_decrypt(key, data, iv, use_padding=True):
@@ -37,4 +37,4 @@ def rc4_encrypt(key, data):
     cipher = Cipher(algorithms.ARC4(key), mode=None)
     encryptor = cipher.encryptor()
     # NOTE: Suppress LGTM warning here, we have to do what the spec says
-    return encryptor.update(data) + encryptor.finalize()  # lgtm
\ No newline at end of file
+    return encryptor.update(data) + encryptor.finalize()  # lgtm
