@@ -1,19 +1,46 @@
 import abc
 import enum
 import logging
+import re
 import secrets
 import struct
+from dataclasses import dataclass
 from hashlib import sha1, sha256
-from typing import Dict, List, Optional, Set, Tuple
+from typing import Dict, List, Optional, Set, Tuple, Union
 
 from asn1crypto import algos, cms, core, x509
-from asn1crypto.keys import PrivateKeyInfo, PublicKeyAlgorithm
-from cryptography.hazmat.primitives import serialization
-from cryptography.hazmat.primitives.asymmetric.padding import PKCS1v15
+from asn1crypto.algos import RSAESOAEPParams
+from asn1crypto.cms import KeyEncryptionAlgorithm, KeyEncryptionAlgorithmId
+from asn1crypto.keys import PrivateKeyInfo, PublicKeyAlgorithm, PublicKeyInfo
+from cryptography.hazmat.primitives import hashes, keywrap, serialization
+from cryptography.hazmat.primitives.asymmetric.ec import (
+    ECDH,
+    EllipticCurvePrivateKey,
+    EllipticCurvePublicKey,
+)
+from cryptography.hazmat.primitives.asymmetric.ec import (
+    generate_private_key as generate_ec_private_key,
+)
+from cryptography.hazmat.primitives.asymmetric.padding import (
+    MGF1,
+    OAEP,
+    AsymmetricPadding,
+    PKCS1v15,
+)
 from cryptography.hazmat.primitives.asymmetric.rsa import (
     RSAPrivateKey,
     RSAPublicKey,
 )
+from cryptography.hazmat.primitives.asymmetric.x448 import (
+    X448PrivateKey,
+    X448PublicKey,
+)
+from cryptography.hazmat.primitives.asymmetric.x25519 import (
+    X25519PrivateKey,
+    X25519PublicKey,
+)
+from cryptography.hazmat.primitives.kdf import KeyDerivationFunction
+from cryptography.hazmat.primitives.kdf.x963kdf import X963KDF
 from cryptography.hazmat.primitives.serialization import pkcs12
 
 from .. import generic, misc
@@ -36,6 +63,22 @@ from .filter_mixins import AESCryptFilterMixin, RC4CryptFilterMixin
 logger = logging.getLogger(__name__)
 
 
+@dataclass(frozen=True)
+class RecipientEncryptionPolicy:
+    ignore_key_usage: bool = False
+    """
+    Ignore key usage bits in the recipient's certificate.
+    """
+
+    prefer_oaep: bool = False
+    """
+    For RSA recipients, encrypt with RSAES-OAEP.
+
+    .. warning::
+        This is not widely supported.
+    """
+
+
 class PubKeyCryptFilter(CryptFilter, abc.ABC):
     """
     Crypt filter for use with public key security handler.
@@ -57,7 +100,7 @@ class PubKeyCryptFilter(CryptFilter, abc.ABC):
             way pyHanko interprets this value.
     """
 
-    _handler: 'PubKeySecurityHandler' = None
+    _handler: Optional['PubKeySecurityHandler'] = None
 
     def __init__(
         self,
@@ -87,8 +130,8 @@ class PubKeyCryptFilter(CryptFilter, abc.ABC):
     def add_recipients(
         self,
         certs: List[x509.Certificate],
+        policy: RecipientEncryptionPolicy,
         perms=ALL_PERMS,
-        ignore_key_usage=False,
     ):
         """
         Add recipients to this crypt filter.
@@ -96,10 +139,10 @@ class PubKeyCryptFilter(CryptFilter, abc.ABC):
 
         :param certs:
             A list of recipient certificates.
+        :param policy:
+            Encryption policy choices for the chosen set of recipients.
         :param perms:
             The permission bits to assign to the listed recipients.
-        :param ignore_key_usage:
-            If ``False``, the *keyEncipherment* key usage extension is required.
         """
 
         if not self.acts_as_default and self.recipients:
@@ -122,8 +165,8 @@ class PubKeyCryptFilter(CryptFilter, abc.ABC):
             certs,
             self._recp_key_seed,
             as_signed(perms),
+            policy=policy,
             include_permissions=self.acts_as_default,
-            ignore_key_usage=ignore_key_usage,
         )
         self.recipients.append(new_cms)
 
@@ -147,6 +190,7 @@ class PubKeyCryptFilter(CryptFilter, abc.ABC):
         return AuthResult(AuthStatus.FAILED)
 
     def derive_shared_encryption_key(self) -> bytes:
+        assert self._handler is not None
         if self._recp_key_seed is None:
             raise misc.PdfError("No seed available; authenticate first.")
         if self._handler.version >= SecurityHandlerVersion.AES256:
@@ -269,47 +313,61 @@ def construct_envelope_content(
     return seed + (struct.pack('<i', perms) if include_permissions else b'')
 
 
-def _recipient_info(
-    envelope_key: bytes, cert: x509.Certificate, ignore_key_usage=False
+def _rsaes_pkcs1v15_recipient(
+    pub_key_info: PublicKeyInfo,
+    rid: cms.RecipientIdentifier,
+    envelope_key: bytes,
 ):
-    pubkey = cert.public_key
-    pubkey_algo_info: PublicKeyAlgorithm = pubkey['algorithm']
-    algorithm_name = pubkey_algo_info['algorithm'].native
-    if algorithm_name != 'rsa':
-        raise NotImplementedError(
-            f"Certificate public key must be of type 'rsa', "
-            f"not '{algorithm_name}'."
-        )
+    padding = PKCS1v15()
 
-    assert len(envelope_key) == 32
-
-    if not ignore_key_usage:
-        key_usage = cert.key_usage_value
-        if key_usage is None or 'key_encipherment' not in key_usage.native:
-            raise misc.PdfWriteError(
-                f"Certificate for subject {cert.subject.human_friendly} does "
-                f"not have the 'key_encipherment' key usage bit set."
-            )
-
-    pub_key = serialization.load_der_public_key(cert.public_key.dump())
-
-    assert isinstance(pub_key, RSAPublicKey)
-    # having support for OAEP here would be cool, but I have it on good
-    #  authority that there's some kind of tacit understanding to use
-    #  PKCS#1 v1.5 padding here.
-    encrypted_data = pub_key.encrypt(envelope_key, padding=PKCS1v15())
-
-    # TODO support subjectKeyIdentifier here (requiring version 2)
-    rid = cms.RecipientIdentifier(
-        {
-            'issuer_and_serial_number': cms.IssuerAndSerialNumber(
-                {'issuer': cert.issuer, 'serial_number': cert.serial_number}
-            )
-        }
-    )
     algo = cms.KeyEncryptionAlgorithm(
         {'algorithm': cms.KeyEncryptionAlgorithmId('rsaes_pkcs1v15')}
     )
+
+    pub_key = serialization.load_der_public_key(pub_key_info.dump())
+    assert isinstance(pub_key, RSAPublicKey)
+    encrypted_data = pub_key.encrypt(envelope_key, padding=padding)
+    return _format_ktri(rid=rid, algo=algo, encrypted_data=encrypted_data)
+
+
+def _rsaes_oaep_recipient(
+    pub_key_info: PublicKeyInfo,
+    rid: cms.RecipientIdentifier,
+    envelope_key: bytes,
+):
+    # recycle these routines
+    from pyhanko.sign.general import get_pyca_cryptography_hash
+    from pyhanko.sign.signers.pdf_cms import select_suitable_signing_md
+
+    digest_function_name = select_suitable_signing_md(pub_key_info)
+    digest_spec = get_pyca_cryptography_hash(digest_function_name)
+    algo = cms.KeyEncryptionAlgorithm(
+        {
+            'algorithm': cms.KeyEncryptionAlgorithmId('rsaes_oaep'),
+            'parameters': RSAESOAEPParams(
+                {
+                    'hash_algorithm': {'algorithm': digest_function_name},
+                    'mask_gen_algorithm': {
+                        'algorithm': 'mgf1',
+                        'parameters': {'algorithm': digest_function_name},
+                    },
+                }
+            ),
+        }
+    )
+    padding = OAEP(mgf=MGF1(digest_spec), algorithm=digest_spec, label=None)
+
+    pub_key = serialization.load_der_public_key(pub_key_info.dump())
+    assert isinstance(pub_key, RSAPublicKey)
+    encrypted_data = pub_key.encrypt(envelope_key, padding=padding)
+    return _format_ktri(rid=rid, algo=algo, encrypted_data=encrypted_data)
+
+
+def _format_ktri(
+    rid: cms.RecipientIdentifier,
+    algo: cms.KeyEncryptionAlgorithm,
+    encrypted_data: bytes,
+):
     return cms.RecipientInfo(
         {
             'ktri': cms.KeyTransRecipientInfo(
@@ -324,12 +382,173 @@ def _recipient_info(
     )
 
 
+def _choose_ecdh_settings(
+    pub_key_info: PublicKeyInfo,
+) -> Tuple[
+    hashes.HashAlgorithm, KeyEncryptionAlgorithmId, KeyEncryptionAlgorithmId
+]:
+    algo_name = pub_key_info.algorithm
+    if algo_name == 'ec':
+        approx_sec_level = pub_key_info.bit_size // 2
+    elif algo_name == 'x25519':
+        approx_sec_level = 128
+    else:
+        approx_sec_level = 256
+    # All of these are dhSinglePass-stdDH-sha*kdf
+    if approx_sec_level <= 128:
+        return (
+            hashes.SHA256(),
+            KeyEncryptionAlgorithmId('aes128_wrap'),
+            KeyEncryptionAlgorithmId('1.3.132.1.11.1'),
+        )
+    elif approx_sec_level <= 192:
+        return (
+            hashes.SHA384(),
+            KeyEncryptionAlgorithmId('aes192_wrap'),
+            KeyEncryptionAlgorithmId('1.3.132.1.11.2'),
+        )
+    else:
+        return (
+            hashes.SHA512(),
+            KeyEncryptionAlgorithmId('aes256_wrap'),
+            KeyEncryptionAlgorithmId('1.3.132.1.11.3'),
+        )
+
+
+def _ecdh_recipient(
+    pub_key_info: PublicKeyInfo,
+    rid: cms.KeyAgreementRecipientIdentifier,
+    envelope_key: bytes,
+):
+    # static-ephemeral ECDH (standard) with X9.63 key derivation
+    digest_spec, key_wrap_algo_id, key_exch_algo_id = _choose_ecdh_settings(
+        pub_key_info
+    )
+    key_wrap_algo = KeyEncryptionAlgorithm({'algorithm': key_wrap_algo_id})
+    pub_key = serialization.load_der_public_key(pub_key_info.dump())
+
+    originator_key: Union[
+        X25519PrivateKey, X448PrivateKey, EllipticCurvePrivateKey
+    ]
+    if isinstance(pub_key, EllipticCurvePublicKey):
+        originator_key = generate_ec_private_key(pub_key.curve)
+        ecdh_value = originator_key.exchange(ECDH(), pub_key)
+    elif isinstance(pub_key, X25519PublicKey):
+        originator_key = X25519PrivateKey.generate()
+        ecdh_value = originator_key.exchange(pub_key)
+    elif isinstance(pub_key, X448PublicKey):
+        originator_key = X448PrivateKey.generate()
+        ecdh_value = originator_key.exchange(pub_key)
+    else:
+        raise NotImplementedError
+
+    originator_key_info = PublicKeyInfo.load(
+        originator_key.public_key().public_bytes(
+            serialization.Encoding.DER,
+            serialization.PublicFormat.SubjectPublicKeyInfo,
+        )
+    )
+
+    ukm = secrets.token_bytes(16)
+    kdf = _kdf_for_exchange(
+        kdf_digest=digest_spec,
+        key_wrap_algo=key_wrap_algo,
+        user_keying_material=ukm,
+    )
+    kek = kdf.derive(ecdh_value)
+    encrypted_data = keywrap.aes_key_wrap(
+        wrapping_key=kek, key_to_wrap=envelope_key
+    )
+    return _format_kari(
+        rid=rid,
+        originator_key=originator_key_info,
+        algo=cms.KeyEncryptionAlgorithm(
+            {
+                'algorithm': key_exch_algo_id,
+                'parameters': key_wrap_algo,
+            }
+        ),
+        ukm=ukm,
+        encrypted_data=encrypted_data,
+    )
+
+
+def _format_kari(
+    rid: cms.KeyAgreementRecipientIdentifier,
+    originator_key: PublicKeyInfo,
+    algo: cms.KeyEncryptionAlgorithm,
+    ukm: bytes,
+    encrypted_data: bytes,
+):
+    return cms.RecipientInfo(
+        name='kari',
+        value=cms.KeyAgreeRecipientInfo(
+            {
+                'version': 3,
+                'originator': cms.OriginatorIdentifierOrKey(
+                    name='originator_key', value=originator_key
+                ),
+                'ukm': ukm,
+                'key_encryption_algorithm': algo,
+                'recipient_encrypted_keys': [
+                    cms.RecipientEncryptedKey(
+                        {'rid': rid, 'encrypted_key': encrypted_data}
+                    )
+                ],
+            }
+        ),
+    )
+
+
+def _recipient_info(
+    envelope_key: bytes,
+    cert: x509.Certificate,
+    policy: RecipientEncryptionPolicy,
+):
+    pub_key_info = cert.public_key
+    pubkey_algo_info: PublicKeyAlgorithm = pub_key_info['algorithm']
+    algorithm_name = pubkey_algo_info['algorithm'].native
+
+    assert len(envelope_key) == 32
+
+    if not policy.ignore_key_usage:
+        # TODO for ECC: reject ECDSA-only keys
+        key_usage = cert.key_usage_value
+        if key_usage is None or 'key_encipherment' not in key_usage.native:
+            raise misc.PdfWriteError(
+                f"Certificate for subject {cert.subject.human_friendly} does "
+                f"not have the 'key_encipherment' key usage bit set."
+            )
+
+    # TODO support subjectKeyIdentifier here (requiring version 2)
+    iss_serial_rid = cms.IssuerAndSerialNumber(
+        {'issuer': cert.issuer, 'serial_number': cert.serial_number}
+    )
+    if algorithm_name == 'rsa':
+        rid = cms.RecipientIdentifier(
+            {'issuer_and_serial_number': iss_serial_rid}
+        )
+        if policy.prefer_oaep:
+            return _rsaes_pkcs1v15_recipient(pub_key_info, rid, envelope_key)
+        else:
+            return _rsaes_oaep_recipient(pub_key_info, rid, envelope_key)
+    elif algorithm_name in ('ec', 'x25519', 'x448'):
+        ka_rid = cms.KeyAgreementRecipientIdentifier(
+            {'issuer_and_serial_number': iss_serial_rid}
+        )
+        return _ecdh_recipient(pub_key_info, ka_rid, envelope_key)
+    else:
+        raise NotImplementedError(
+            f"Cannot encrypt for key type '{algorithm_name}'"
+        )
+
+
 def construct_recipient_cms(
     certificates: List[x509.Certificate],
     seed: bytes,
     perms: int,
+    policy: RecipientEncryptionPolicy,
     include_permissions=True,
-    ignore_key_usage=False,
 ) -> cms.ContentInfo:
     # The content of the generated ContentInfo object
     # is an object of type EnvelopedData, containing a 20 byte seed (+ perms).
@@ -358,7 +577,7 @@ def construct_recipient_cms(
 
     # encrypt the envelope key for each recipient
     rec_infos = [
-        _recipient_info(envelope_key, cert, ignore_key_usage=ignore_key_usage)
+        _recipient_info(envelope_key, cert, policy=policy)
         for cert in certificates
     ]
 
@@ -377,7 +596,7 @@ def construct_recipient_cms(
     )
 
     # version 0 because no originatorInfo, no attribute certs
-    # and all recipientinfo structures have version 0 (and aren't' pwri)
+    # and all recipientinfo structures have version 0 (and aren't pwri)
     enveloped_data = cms.EnvelopedData(
         {
             'version': 0,
@@ -395,6 +614,10 @@ def construct_recipient_cms(
     )
 
 
+class InappropriateCredentialError(TypeError):
+    pass
+
+
 # TODO implement a PKCS#11 version of this interface
 class EnvelopeKeyDecrypter:
     """
@@ -402,24 +625,53 @@ class EnvelopeKeyDecrypter:
 
     This allows the key decryption process to happen offline, e.g. on a smart
     card.
-
-    :param cert:
-        The recipient's certificate.
     """
 
-    def __init__(self, cert: x509.Certificate):
-        self.cert = cert
+    @property
+    def cert(self) -> x509.Certificate:
+        """
+        :return:
+            Return the recipient's certificate
+        """
+        raise NotImplementedError
 
     def decrypt(
         self, encrypted_key: bytes, algo_params: cms.KeyEncryptionAlgorithm
     ) -> bytes:
         """
         Invoke the actual key decryption algorithm.
+        Used with key transport.
 
         :param encrypted_key:
             Payload to decrypt.
         :param algo_params:
             Specification of the encryption algorithm as a CMS object.
+        :raises InappropriateCredentialError:
+            if the credential cannot be used for key transport.
+        :return:
+            The decrypted payload.
+        """
+        raise NotImplementedError
+
+    def decrypt_with_exchange(
+        self,
+        encrypted_key: bytes,
+        algo_params: cms.KeyEncryptionAlgorithm,
+        originator_identifier: cms.OriginatorIdentifierOrKey,
+        user_keying_material: bytes,
+    ) -> bytes:
+        """
+        Decrypt an envelope key using a key derived from a key exchange.
+
+        :param encrypted_key:
+            Payload to decrypt.
+        :param algo_params:
+            Specification of the encryption algorithm as a CMS object.
+        :param originator_identifier:
+            Information about the originator necessary to complete the key
+            exchange.
+        :param user_keying_material:
+            The user keying material that will be used in the key derivation.
         :return:
             The decrypted payload.
         """
@@ -430,10 +682,51 @@ class _PrivKeyAndCert(core.Sequence):
     _fields = [('key', PrivateKeyInfo), ('cert', x509.Certificate)]
 
 
+class ECCCMSSharedInfo(core.Sequence):
+    _fields = [
+        ('key_info', KeyEncryptionAlgorithm),
+        (
+            'entityUInfo',
+            core.OctetString,
+            {'explicit': 0, 'optional': True},
+        ),
+        ('suppPubInfo', core.OctetString, {'explicit': 2}),
+    ]
+
+
+AES_WRAP_PATTERN = re.compile(r'aes(\d+)_wrap(_pad)?')
+
+
+def _kdf_for_exchange(
+    *,
+    kdf_digest: hashes.HashAlgorithm,
+    key_wrap_algo: cms.KeyEncryptionAlgorithm,
+    user_keying_material: Optional[bytes],
+) -> KeyDerivationFunction:
+    key_wrap_algo_id: str = key_wrap_algo['algorithm'].native
+    wrap_match = AES_WRAP_PATTERN.fullmatch(key_wrap_algo_id)
+    if not wrap_match:
+        raise NotImplementedError(
+            f"{key_wrap_algo_id} is not a supported key wrapping algorithm"
+        )
+    kek_bit_len = int(wrap_match.group(1))
+    return X963KDF(
+        algorithm=kdf_digest,
+        length=kek_bit_len // 8,
+        sharedinfo=ECCCMSSharedInfo(
+            {
+                'key_info': key_wrap_algo,
+                'entityUInfo': user_keying_material,
+                'suppPubInfo': struct.pack('>I', kek_bit_len),
+            }
+        ).dump(),
+    )
+
+
 class SimpleEnvelopeKeyDecrypter(EnvelopeKeyDecrypter, SerialisableCredential):
     """
     Implementation of :class:`.EnvelopeKeyDecrypter` where the private key
-    is an RSA key residing in memory.
+    is an RSA or ECC key residing in memory.
 
     :param cert:
         The recipient's certificate.
@@ -441,6 +734,8 @@ class SimpleEnvelopeKeyDecrypter(EnvelopeKeyDecrypter, SerialisableCredential):
         The recipient's private key.
     """
 
+    dhsinglepass_stddh_arc_pattern = re.compile(r'1\.3\.132\.1\.11\.(\d+)')
+
     @classmethod
     def get_name(cls) -> str:
         return 'raw_privkey'
@@ -462,8 +757,12 @@ class SimpleEnvelopeKeyDecrypter(EnvelopeKeyDecrypter, SerialisableCredential):
         return SimpleEnvelopeKeyDecrypter(cert=cert, private_key=key)
 
     def __init__(self, cert: x509.Certificate, private_key: PrivateKeyInfo):
-        super().__init__(cert)
         self.private_key: PrivateKeyInfo = private_key
+        self._cert = cert
+
+    @property
+    def cert(self) -> x509.Certificate:
+        return self._cert
 
     @staticmethod
     def load(key_file, cert_file, key_passphrase=None):
@@ -479,13 +778,13 @@ class SimpleEnvelopeKeyDecrypter(EnvelopeKeyDecrypter, SerialisableCredential):
         :return:
             An instance of :class:`.SimpleEnvelopeKeyDecrypter`.
         """
-        from pyhanko.sign.general import load_private_key_from_pemder
+        from ...keys import load_private_key_from_pemder
 
         try:
             private_key = load_private_key_from_pemder(
                 key_file, passphrase=key_passphrase
             )
-            from pyhanko.sign.general import load_cert_from_pemder
+            from ...keys import load_cert_from_pemder
 
             cert = load_cert_from_pemder(cert_file)
         except (IOError, ValueError, TypeError) as e:  # pragma: nocover
@@ -513,7 +812,7 @@ class SimpleEnvelopeKeyDecrypter(EnvelopeKeyDecrypter, SerialisableCredential):
                 pfx_bytes, passphrase
             )
 
-            from ...sign.general import (
+            from ...keys import (
                 _translate_pyca_cryptography_cert_to_asn1,
                 _translate_pyca_cryptography_key_to_asn1,
             )
@@ -530,32 +829,232 @@ class SimpleEnvelopeKeyDecrypter(EnvelopeKeyDecrypter, SerialisableCredential):
         self, encrypted_key: bytes, algo_params: cms.KeyEncryptionAlgorithm
     ) -> bytes:
         """
-        Decrypt the payload using RSA with PKCS#1 v1.5 padding.
+        Decrypt the payload using RSA with PKCS#1 v1.5 padding or OAEP.
         Other schemes are not (currently) supported by this implementation.
 
         :param encrypted_key:
             Payload to decrypt.
         :param algo_params:
             Specification of the encryption algorithm as a CMS object.
-            Must use ``rsaes_pkcs1v15``.
+            Must use ``rsaes_pkcs1v15`` or ``rsaes_oaep``.
         :return:
             The decrypted payload.
         """
         algo_name = algo_params['algorithm'].native
-        if algo_name != 'rsaes_pkcs1v15':
-            raise NotImplementedError(
-                f"Only 'rsaes_pkcs1v15' is supported for envelope encryption, "
-                f"not '{algo_name}'."
+        padding: AsymmetricPadding
+        if algo_name == 'rsaes_pkcs1v15':
+            padding = PKCS1v15()
+        elif algo_name == 'rsaes_oaep':
+            from pyhanko.sign.general import get_pyca_cryptography_hash
+
+            oaep_params: RSAESOAEPParams = algo_params['parameters']
+            mgf = oaep_params['mask_gen_algorithm']
+            mgf_name = mgf['algorithm'].native
+            if mgf_name != 'mgf1':
+                raise NotImplementedError(
+                    f"Only MGF1 is implemented, but got '{mgf_name}'"
+                )
+            padding = OAEP(
+                mgf=MGF1(
+                    algorithm=get_pyca_cryptography_hash(
+                        mgf['parameters']['algorithm'].native
+                    )
+                ),
+                algorithm=get_pyca_cryptography_hash(
+                    oaep_params['hash_algorithm']['algorithm'].native
+                ),
+                label=None,
             )
-        priv_key: RSAPrivateKey = serialization.load_der_private_key(
+        else:
+            raise NotImplementedError(
+                f"Only 'rsaes_pkcs1v15' and 'rsaes_oaep' are supported for "
+                f"envelope decryption, not '{algo_name}'."
+            )
+        priv_key = serialization.load_der_private_key(
             self.private_key.dump(), password=None
         )
-        return priv_key.decrypt(encrypted_key, padding=PKCS1v15())
+        if not isinstance(priv_key, RSAPrivateKey):
+            raise InappropriateCredentialError(
+                "The loaded key does not seem to be an RSA private key"
+            )
+        return priv_key.decrypt(encrypted_key, padding=padding)
+
+    def decrypt_with_exchange(
+        self,
+        encrypted_key: bytes,
+        algo_params: cms.KeyEncryptionAlgorithm,
+        originator_identifier: cms.OriginatorIdentifierOrKey,
+        user_keying_material: Optional[bytes],
+    ) -> bytes:
+        """
+        Decrypt the payload using a key agreed via ephemeral-static
+        standard (non-cofactor) ECDH with X9.63 key derivation.
+        Other schemes aer not supported at this time.
+
+
+        :param encrypted_key:
+            Payload to decrypt.
+        :param algo_params:
+            Specification of the encryption algorithm as a CMS object.
+        :param originator_identifier:
+            The originator info, which must be an EC key.
+        :param user_keying_material:
+            The user keying material that will be used in the key derivation.
+        :return:
+            The decrypted payload.
+        """
+        # TODO get the relevant OIDs added to asn1crypto
+        oid = algo_params['algorithm']
+
+        match = self.dhsinglepass_stddh_arc_pattern.fullmatch(oid.dotted)
+        kdf_digest: Optional[hashes.HashAlgorithm] = None
+        if match:
+            kdf_digest = {
+                '0': hashes.SHA224(),
+                '1': hashes.SHA256(),
+                '2': hashes.SHA384(),
+                '3': hashes.SHA512(),
+            }.get(match.group(1), None)
+        if not kdf_digest:
+            raise NotImplementedError(
+                "Only dhSinglePass-stdDH algorithms from SEC 1 / RFC 5753 are "
+                "supported."
+            )
+
+        key_wrap_algo = cms.KeyEncryptionAlgorithm.load(
+            algo_params['parameters'].dump()
+        )
+        key_wrap_algo_id: str = key_wrap_algo['algorithm'].native
+
+        if not AES_WRAP_PATTERN.fullmatch(key_wrap_algo_id):
+            raise NotImplementedError(
+                f"{key_wrap_algo_id} is not a supported key wrapping algorithm"
+            )
+        unwrap_key = (
+            keywrap.aes_key_unwrap_with_padding
+            if key_wrap_algo_id.endswith('_pad')
+            else keywrap.aes_key_unwrap
+        )
+        kdf = _kdf_for_exchange(
+            kdf_digest=kdf_digest,
+            key_wrap_algo=key_wrap_algo,
+            user_keying_material=user_keying_material,
+        )
+
+        if originator_identifier.name != 'originator_key':
+            raise NotImplementedError("Only originator_key is supported")
+
+        originator_pub_key_info: PublicKeyInfo = (
+            originator_identifier.chosen.untag()
+        )
+        originator_pub_key = serialization.load_der_public_key(
+            originator_pub_key_info.dump()
+        )
+
+        priv_key = serialization.load_der_private_key(
+            self.private_key.dump(), password=None
+        )
+
+        mismatch_msg = (
+            "Originator's public key is not compatible "
+            "with selected private key"
+        )
+        if isinstance(priv_key, EllipticCurvePrivateKey):
+            # we could rely on pyca/cryptography to perform this check for us,
+            #  but then mypy complains
+            if (
+                not isinstance(originator_pub_key, EllipticCurvePublicKey)
+                or originator_pub_key.curve.name != priv_key.curve.name
+            ):
+                raise ValueError(mismatch_msg)
+            ecdh_value = priv_key.exchange(
+                ECDH(), peer_public_key=originator_pub_key
+            )
+        elif isinstance(priv_key, X25519PrivateKey):
+            if not isinstance(originator_pub_key, X25519PublicKey):
+                raise ValueError(mismatch_msg)
+            ecdh_value = priv_key.exchange(originator_pub_key)
+        elif isinstance(priv_key, X448PrivateKey):
+            if not isinstance(originator_pub_key, X448PublicKey):
+                raise ValueError(mismatch_msg)
+            ecdh_value = priv_key.exchange(originator_pub_key)
+        else:
+            raise InappropriateCredentialError(
+                "The loaded key does not seem to be an EC private key"
+            )
+        derived_kek = kdf.derive(ecdh_value)
+        return unwrap_key(wrapping_key=derived_kek, wrapped_key=encrypted_key)
 
 
 SerialisableCredential.register(SimpleEnvelopeKeyDecrypter)
 
 
+def read_envelope_key(
+    ed: cms.EnvelopedData, decrypter: EnvelopeKeyDecrypter
+) -> Optional[bytes]:
+    rec_info: cms.RecipientInfo
+    for rec_info in ed['recipient_infos']:
+        if rec_info.name == 'ktri':
+            ktri = rec_info.chosen
+            issuer_and_serial = ktri['rid'].chosen
+            if not isinstance(issuer_and_serial, cms.IssuerAndSerialNumber):
+                raise NotImplementedError(
+                    "Recipient identifier must be of type IssuerAndSerialNumber."
+                )
+            issuer = issuer_and_serial['issuer']
+            serial = issuer_and_serial['serial_number'].native
+            if (
+                decrypter.cert.issuer == issuer
+                and decrypter.cert.serial_number == serial
+            ):
+                try:
+                    return decrypter.decrypt(
+                        ktri['encrypted_key'].native,
+                        ktri['key_encryption_algorithm'],
+                    )
+                except InappropriateCredentialError as e:
+                    raise e
+                except Exception as e:
+                    raise misc.PdfReadError(
+                        "Failed to decrypt envelope key"
+                    ) from e
+        elif rec_info.name == 'kari':
+            kari = rec_info.chosen
+            for recipient_enc_key in kari['recipient_encrypted_keys']:
+                issuer_and_serial = recipient_enc_key['rid'].chosen
+                # TODO make this DRY and support key identifier
+                if not isinstance(issuer_and_serial, cms.IssuerAndSerialNumber):
+                    raise NotImplementedError(
+                        "Recipient identifier must be of type IssuerAndSerialNumber."
+                    )
+                issuer = issuer_and_serial['issuer']
+                serial = issuer_and_serial['serial_number'].native
+                if (
+                    decrypter.cert.issuer == issuer
+                    and decrypter.cert.serial_number == serial
+                ):
+                    try:
+                        return decrypter.decrypt_with_exchange(
+                            recipient_enc_key['encrypted_key'].native,
+                            kari['key_encryption_algorithm'],
+                            originator_identifier=kari['originator'],
+                            user_keying_material=kari['ukm'].native,
+                        )
+                    except InappropriateCredentialError as e:
+                        raise e
+                    except Exception as e:
+                        raise misc.PdfReadError(
+                            "Failed to decrypt envelope key"
+                        ) from e
+        else:
+            raise NotImplementedError(
+                "RecipientInfo must be of type KeyTransRecipientInfo "
+                "or KeyAgreeRecipientInfo"
+            )
+
+    return None
+
+
 def read_seed_from_recipient_cms(
     recipient_cms: cms.ContentInfo, decrypter: EnvelopeKeyDecrypter
 ) -> Tuple[Optional[bytes], Optional[int]]:
@@ -567,36 +1066,8 @@ def read_seed_from_recipient_cms(
         )
     ed: cms.EnvelopedData = recipient_cms['content']
     encrypted_content_info = ed['encrypted_content_info']
-    rec_info: cms.RecipientInfo
-    for rec_info in ed['recipient_infos']:
-        ktri = rec_info.chosen
-        if not isinstance(ktri, cms.KeyTransRecipientInfo):
-            raise NotImplementedError(
-                "RecipientInfo must be of type KeyTransRecipientInfo."
-            )
-        issuer_and_serial = ktri['rid'].chosen
-        if not isinstance(issuer_and_serial, cms.IssuerAndSerialNumber):
-            raise NotImplementedError(
-                "Recipient identifier must be of type IssuerAndSerialNumber."
-            )
-        issuer = issuer_and_serial['issuer']
-        serial = issuer_and_serial['serial_number'].native
-        if (
-            decrypter.cert.issuer == issuer
-            and decrypter.cert.serial_number == serial
-        ):
-            # we have a match!
-            # use the decrypter passed in to decrypt the envelope key
-            # for this recipient.
-            try:
-                envelope_key = decrypter.decrypt(
-                    ktri['encrypted_key'].native,
-                    ktri['key_encryption_algorithm'],
-                )
-            except Exception as e:
-                raise misc.PdfReadError("Failed to decrypt envelope key") from e
-            break
-    else:
+    envelope_key = read_envelope_key(ed, decrypter)
+    if envelope_key is None:
         return None, None
 
     # we have the envelope key
@@ -709,10 +1180,10 @@ class PubKeySecurityHandler(SecurityHandler):
     """
 
     _known_crypt_filters: Dict[generic.NameObject, CryptFilterBuilder] = {
-        '/V2': _build_legacy_pubkey_cf,
-        '/AESV2': _build_aes128_pubkey_cf,
-        '/AESV3': _build_aes256_pubkey_cf,
-        '/Identity': lambda _, __: IdentityCryptFilter(),
+        generic.NameObject('/V2'): _build_legacy_pubkey_cf,
+        generic.NameObject('/AESV2'): _build_aes128_pubkey_cf,
+        generic.NameObject('/AESV3'): _build_aes256_pubkey_cf,
+        generic.NameObject('/Identity'): lambda _, __: IdentityCryptFilter(),
     }
 
     @classmethod
@@ -725,7 +1196,7 @@ class PubKeySecurityHandler(SecurityHandler):
         use_crypt_filters=True,
         perms: int = ALL_PERMS,
         encrypt_metadata=True,
-        ignore_key_usage=False,
+        policy: RecipientEncryptionPolicy = RecipientEncryptionPolicy(),
         **kwargs,
     ) -> 'PubKeySecurityHandler':
         """
@@ -758,8 +1229,8 @@ class PubKeySecurityHandler(SecurityHandler):
             .. warning::
                 See :class:`.SecurityHandler` for some background on the
                 way pyHanko interprets this value.
-        :param ignore_key_usage:
-            If ``False``, the *keyEncipherment* key usage extension is required.
+        :param policy:
+            Encryption policy choices for the chosen set of recipients.
         :return:
             An instance of :class:`.PubKeySecurityHandler`.
         """
@@ -792,7 +1263,7 @@ class PubKeySecurityHandler(SecurityHandler):
             recipient_objs=None,
             **kwargs,
         )
-        sh.add_recipients(certs, perms=perms, ignore_key_usage=ignore_key_usage)
+        sh.add_recipients(certs, perms=perms, policy=policy)
         return sh
 
     def __init__(
@@ -801,8 +1272,8 @@ class PubKeySecurityHandler(SecurityHandler):
         pubkey_handler_subfilter: PubKeyAdbeSubFilter,
         legacy_keylen,
         encrypt_metadata=True,
-        crypt_filter_config: 'CryptFilterConfiguration' = None,
-        recipient_objs: list = None,
+        crypt_filter_config: Optional['CryptFilterConfiguration'] = None,
+        recipient_objs: Optional[list] = None,
         compat_entries=True,
     ):
         # I don't see how it would be possible to handle V4 without
@@ -838,6 +1309,10 @@ class PubKeySecurityHandler(SecurityHandler):
                     encrypt_metadata=encrypt_metadata,
                     recipients=recipient_objs,
                 )
+            else:
+                raise misc.PdfError(
+                    "Failed to impute a reasonable crypt filter config"
+                )
         super().__init__(
             version,
             legacy_keylen,
@@ -977,7 +1452,7 @@ class PubKeySecurityHandler(SecurityHandler):
         self,
         certs: List[x509.Certificate],
         perms=ALL_PERMS,
-        ignore_key_usage=False,
+        policy: RecipientEncryptionPolicy = RecipientEncryptionPolicy(),
     ):
         # add recipients to all *default* crypt filters
         # callers that want to do this more granularly are welcome to, but
@@ -986,12 +1461,12 @@ class PubKeySecurityHandler(SecurityHandler):
         for cf in self.crypt_filter_config.standard_filters():
             if not isinstance(cf, PubKeyCryptFilter):
                 continue
-            cf.add_recipients(
-                certs, perms=perms, ignore_key_usage=ignore_key_usage
-            )
+            cf.add_recipients(certs, perms=perms, policy=policy)
 
     def authenticate(
-        self, credential: EnvelopeKeyDecrypter, id1=None
+        self,
+        credential: Union[EnvelopeKeyDecrypter, SerialisedCredential],
+        id1=None,
     ) -> AuthResult:
         """
         Authenticate a user to this security handler.
@@ -1007,20 +1482,24 @@ class PubKeySecurityHandler(SecurityHandler):
             obtained.
         """
 
+        actual_credential: EnvelopeKeyDecrypter
         if isinstance(credential, SerialisedCredential):
-            credential = SerialisableCredential.deserialise(credential)
-        if not isinstance(credential, EnvelopeKeyDecrypter):
-            raise misc.PdfReadError(
-                f"Pubkey authentication credential must be an instance of "
-                f"EnvelopeKeyDecrypter, not {type(credential)}."
-            )
+            deser_credential = SerialisableCredential.deserialise(credential)
+            if not isinstance(deser_credential, EnvelopeKeyDecrypter):
+                raise misc.PdfReadError(
+                    f"Pubkey authentication credential must be an instance of "
+                    f"EnvelopeKeyDecrypter, not {type(deser_credential)}."
+                )
+            actual_credential = deser_credential
+        else:
+            actual_credential = credential
 
         perms = 0xFFFFFFFF
         for cf in self.crypt_filter_config.standard_filters():
             if not isinstance(cf, PubKeyCryptFilter):
                 continue
             recp: cms.ContentInfo
-            result = cf.authenticate(credential)
+            result = cf.authenticate(actual_credential)
             if result.status == AuthStatus.FAILED:
                 return result
             # these should really be the same for both filters, but hey,
@@ -1028,8 +1507,8 @@ class PubKeySecurityHandler(SecurityHandler):
             # course of action
             if result.permission_flags is not None:
                 perms &= result.permission_flags
-        if isinstance(credential, SerialisableCredential):
-            self._credential = credential
+        if isinstance(actual_credential, SerialisableCredential):
+            self._credential = actual_credential
         return AuthResult(AuthStatus.USER, as_signed(perms))
 
     def get_file_encryption_key(self) -> bytes:
