@@ -12,12 +12,12 @@ import enum
 import logging
 import os
 import re
+import typing
 from dataclasses import dataclass, field
 from datetime import datetime, timedelta, timezone
 from io import BytesIO
 from typing import Any, Callable, Iterator, Optional, Tuple, Union
 
-from . import filters
 from .misc import (
     IndirectObjectExpected,
     PdfError,
@@ -32,6 +32,9 @@ from .misc import (
     skip_over_whitespace,
 )
 
+if typing.TYPE_CHECKING:
+    from .crypt.api import SecurityHandler
+
 __all__ = [
     'Dereferenceable',
     'Reference',
@@ -283,7 +286,7 @@ def read_object(
 class PdfObject:
     """Superclass for all PDF objects."""
 
-    container_ref: Dereferenceable = None
+    container_ref: Optional[Dereferenceable] = None
     """
     For objects read from a file, `container_ref` points to the unique
     addressable object containing this object.
@@ -331,7 +334,10 @@ class PdfObject:
         return self
 
     def write_to_stream(
-        self, stream, handler=None, container_ref: Reference = None
+        self,
+        stream,
+        handler: Optional['SecurityHandler'] = None,
+        container_ref: Optional[Reference] = None,
     ):
         """
         Abstract method to render this object to an output stream.
@@ -353,7 +359,12 @@ class NullObject(PdfObject):
     All instances are treated as equal and falsy.
     """
 
-    def write_to_stream(self, stream, handler=None, container_ref=None):
+    def write_to_stream(
+        self,
+        stream,
+        handler: Optional['SecurityHandler'] = None,
+        container_ref=None,
+    ):
         stream.write(b"null")
 
     @staticmethod
@@ -379,7 +390,12 @@ class BooleanObject(PdfObject):
     def __init__(self, value):
         self.value = value
 
-    def write_to_stream(self, stream, handler=None, container_ref=None):
+    def write_to_stream(
+        self,
+        stream,
+        handler: Optional['SecurityHandler'] = None,
+        container_ref=None,
+    ):
         if self.value:
             stream.write(b"true")
         else:
@@ -391,10 +407,9 @@ class BooleanObject(PdfObject):
         if word == b"true":
             return BooleanObject(True)
         elif word == b"fals":
-            stream.read(1)
-            return BooleanObject(False)
-        else:
-            raise PdfReadError('Could not read Boolean object')
+            if stream.read(1) == b"e":
+                return BooleanObject(False)
+        raise PdfReadError('Could not read Boolean object')
 
     def __bool__(self):
         return bool(self.value)
@@ -459,7 +474,12 @@ class ArrayObject(list, PdfObject):
         val = list.__getitem__(self, index)
         return _deproxy_decrypt(val, decrypt)
 
-    def write_to_stream(self, stream, handler=None, container_ref=None):
+    def write_to_stream(
+        self,
+        stream,
+        handler: Optional['SecurityHandler'] = None,
+        container_ref=None,
+    ):
         stream.write(b"[")
         for data in self:
             stream.write(b" ")
@@ -548,39 +568,55 @@ class IndirectObject(PdfObject, Dereferenceable):
     def __ne__(self, other):
         return not self.__eq__(other)
 
-    def write_to_stream(self, stream, handler=None, container_ref=None):
+    def write_to_stream(
+        self,
+        stream,
+        handler: Optional['SecurityHandler'] = None,
+        container_ref=None,
+    ):
         stream.write(b"%d %d R" % (self.idnum, self.generation))
 
     @staticmethod
     def read_from_stream(stream, container_ref: 'Dereferenceable'):
-        idnum = b""
+        idnum_str = b""
         while True:
             tok = stream.read(1)
             if not tok:
                 # stream has truncated prematurely
                 raise PdfStreamError("Stream has ended unexpectedly")
             if tok.isspace():
-                break
-            idnum += tok
-        generation = b""
-        while True:
-            tok = stream.read(1)
-            if not tok:
-                # stream has truncated prematurely
-                raise PdfStreamError("Stream has ended unexpectedly")
-            if tok.isspace():
-                if not generation:
+                if not idnum_str:
                     continue
                 break
-            generation += tok
+            idnum_str += tok
+        generation_str = b""
+        while True:
+            tok = stream.read(1)
+            if not tok:
+                # stream has truncated prematurely
+                raise PdfStreamError("Stream has ended unexpectedly")
+            if tok.isspace():
+                if not generation_str:
+                    continue
+                break
+            generation_str += tok
         r = read_non_whitespace(stream)
         if r != b"R":
             pos = hex(stream.tell())
             raise PdfReadError(
                 "Error reading indirect object reference at byte %s" % pos
             )
+        try:
+            idnum, generation = int(idnum_str), int(generation_str)
+            if not (idnum > 0 and generation >= 0):
+                raise ValueError
+        except ValueError:
+            pos = hex(stream.tell())
+            raise PdfReadError(
+                f"Parse error on indirect object reference around {pos}"
+            )
         return IndirectObject(
-            int(idnum), int(generation), container_ref.get_pdf_handler()
+            int(idnum_str), int(generation_str), container_ref.get_pdf_handler()
         )
 
 
@@ -592,12 +628,8 @@ class FloatObject(decimal.Decimal, PdfObject):
     fixed-point objects, to be precise).
     """
 
-    # noinspection PyArgumentList,PyTypeChecker
-    def __new__(cls, value="0", context=None):
-        try:
-            return decimal.Decimal.__new__(cls, str(value), context)
-        except (ValueError, decimal.DecimalException):
-            return decimal.Decimal.__new__(cls, str(value))
+    def __new__(cls, value="0"):
+        return decimal.Decimal.__new__(cls, str(value))
 
     def __repr__(self):
         if self == self.to_integral():
@@ -611,7 +643,12 @@ class FloatObject(decimal.Decimal, PdfObject):
         """
         return float(self)
 
-    def write_to_stream(self, stream, handler=None, container_ref=None):
+    def write_to_stream(
+        self,
+        stream,
+        handler: Optional['SecurityHandler'] = None,
+        container_ref=None,
+    ):
         stream.write(repr(self).encode('ascii'))
 
 
@@ -626,10 +663,7 @@ class NumberObject(int, PdfObject):
     # noinspection PyArgumentList
     def __new__(cls, value):
         val = int(value)
-        try:
-            return int.__new__(cls, val)
-        except OverflowError:
-            return int.__new__(cls, 0)
+        return int.__new__(cls, val)
 
     def as_numeric(self):
         """
@@ -637,7 +671,12 @@ class NumberObject(int, PdfObject):
         """
         return int(self)
 
-    def write_to_stream(self, stream, handler=None, container_ref=None):
+    def write_to_stream(
+        self,
+        stream,
+        handler: Optional['SecurityHandler'] = None,
+        container_ref=None,
+    ):
         stream.write(repr(self).encode('ascii'))
 
     @staticmethod
@@ -801,8 +840,13 @@ class ByteStringObject(bytes, PdfObject):
     For compatibility with :attr:`.TextStringObject.original_bytes`
     """
 
-    def write_to_stream(self, stream, handler=None, container_ref=None):
-        bytearr = self
+    def write_to_stream(
+        self,
+        stream,
+        handler: Optional['SecurityHandler'] = None,
+        container_ref=None,
+    ):
+        bytearr: bytes = self
         if handler is not None and container_ref is not None:
             cf = handler.get_string_filter()
             local_key = cf.derive_object_key(
@@ -928,14 +972,19 @@ class TextStringObject(str, PdfObject):
         else:
             raise PdfError("No information about original bytes")
 
-    def write_to_stream(self, stream, handler=None, container_ref=None):
+    def write_to_stream(
+        self,
+        stream,
+        handler: Optional['SecurityHandler'] = None,
+        container_ref=None,
+    ):
+        encoded: bytes
         if self.force_output_encoding is not None:
             encoded = self.force_output_encoding.encode(self)
         else:
             # Try to write the string out as a PDFDocEncoding encoded string.
             # It's nicer to look at in the PDF file.  Sadly, we take a
             # performance hit here for trying...
-            encoded: bytes
             try:
                 encoded = encode_pdfdocencoding(self)
             except UnicodeEncodeError:
@@ -1041,7 +1090,12 @@ class NameObject(str, PdfObject):
     required.
     """
 
-    def write_to_stream(self, stream, handler=None, container_ref=None):
+    def write_to_stream(
+        self,
+        stream,
+        handler: Optional['SecurityHandler'] = None,
+        container_ref=None,
+    ):
         byte_iter = iter(self.encode('utf8'))
         if not next(byte_iter) == 0x2F:
             raise PdfWriteError(
@@ -1075,7 +1129,7 @@ def _normalise_key(key):
         if isinstance(key, str):
             return NameObject(key)
         else:
-            raise ValueError("key must be PdfName")
+            raise ValueError("key must be a name object")
     return key
 
 
@@ -1098,7 +1152,9 @@ class DictionaryObject(dict, PdfObject):
             super().__init__()
 
     def raw_get(
-        self, key, decrypt: EncryptedObjAccess = EncryptedObjAccess.TRANSPARENT
+        self,
+        key: Union[NameObject, str],
+        decrypt: EncryptedObjAccess = EncryptedObjAccess.TRANSPARENT,
     ):
         """
         .. versionchanged:: 0.14.0
@@ -1148,7 +1204,11 @@ class DictionaryObject(dict, PdfObject):
                 raw_obj.reference, as_metadata_stream=True
             )
         else:
-            return raw_obj.get_object()
+            deref_obj = raw_obj.get_object()
+            if isinstance(deref_obj, NullObject):
+                raise KeyError(key)
+            else:
+                return deref_obj
 
     def get_and_apply(
         self,
@@ -1175,7 +1235,12 @@ class DictionaryObject(dict, PdfObject):
             raise KeyError
         return value
 
-    def write_to_stream(self, stream, handler=None, container_ref=None):
+    def write_to_stream(
+        self,
+        stream,
+        handler: Optional['SecurityHandler'] = None,
+        container_ref=None,
+    ):
         stream.write(b"<<\n")
         for key, value in list(self.items()):
             key.write_to_stream(stream, handler, container_ref)
@@ -1316,7 +1381,11 @@ class StreamObject(DictionaryObject):
     """
 
     def __init__(
-        self, dict_data=None, stream_data=None, encoded_data=None, handler=None
+        self,
+        dict_data: Optional[dict] = None,
+        stream_data: Optional[bytes] = None,
+        encoded_data: Optional[bytes] = None,
+        handler: Optional['SecurityHandler'] = None,
     ):
         super().__init__(dict_data)
         self._data = stream_data
@@ -1366,11 +1435,17 @@ class StreamObject(DictionaryObject):
         return '/Crypt' in (name for name, _ in self._filters())
 
     def add_crypt_filter(
-        self, name=NameObject('/Identity'), params=None, handler=None
+        self,
+        name=NameObject('/Identity'),
+        params=None,
+        handler: Optional['SecurityHandler'] = None,
     ):
         if handler is not None:
             self._handler = handler
 
+        if self._handler is None:
+            raise PdfStreamError("There is no security handler around")
+
         if name not in self._handler.crypt_filter_config:
             raise PdfStreamError(
                 f"The crypt filter {name} is not known to the security handler."
@@ -1392,7 +1467,7 @@ class StreamObject(DictionaryObject):
             # we have a single filter instance
             filter_arr = (filter_arr,)
         elif not isinstance(filter_arr, ArrayObject):
-            raise TypeError(
+            raise PdfStreamError(
                 '/Filter should be a name object or an array of names.'
             )
 
@@ -1400,8 +1475,8 @@ class StreamObject(DictionaryObject):
             decode_params = self[pdf_name('/DecodeParms')]
             if isinstance(decode_params, DictionaryObject):
                 # one instance
-                decode_params = (decode_params,)
-            if isinstance(decode_params, (ArrayObject, tuple)):
+                decode_params = [decode_params]
+            if isinstance(decode_params, list):
                 lendiff = len(filter_arr) - len(decode_params)
                 # this should be zero, but let's be lenient
                 if lendiff > 0:
@@ -1416,6 +1491,8 @@ class StreamObject(DictionaryObject):
         )
 
     def _stream_decoders(self):
+        from . import filters
+
         for filter_type, params in self._filters():
             try:
                 if params is None or isinstance(params, NullObject):
@@ -1469,6 +1546,7 @@ class StreamObject(DictionaryObject):
             if isinstance(data, memoryview):
                 data = data.tobytes()
             self._data = data
+        assert self._data is not None
         return self._data
 
     @property
@@ -1488,6 +1566,7 @@ class StreamObject(DictionaryObject):
             for filter_cls, decode_params in reversed(decoders):
                 data = filter_cls.encode(data, decode_params)
             self._encoded_data = data
+        assert self._encoded_data is not None
         return self._encoded_data
 
     def apply_filter(
@@ -1578,7 +1657,12 @@ class StreamObject(DictionaryObject):
         except KeyError:
             return False
 
-    def write_to_stream(self, stream, handler=None, container_ref=None):
+    def write_to_stream(
+        self,
+        stream,
+        handler: Optional['SecurityHandler'] = None,
+        container_ref=None,
+    ):
         data = self.encoded_data
         if (
             handler is not None
@@ -1943,7 +2027,7 @@ class DecryptedObjectProxy(PdfObject):
 
     def __init__(self, raw_object: PdfObject, handler):
         self.raw_object = raw_object
-        self._decrypted = None
+        self._decrypted: Optional[PdfObject] = None
         self.handler = handler
 
     @property
@@ -1957,12 +2041,13 @@ class DecryptedObjectProxy(PdfObject):
         method on the container.
         """
 
-        decrypted = self._decrypted
-        if decrypted is not None:
-            return decrypted
+        if self._decrypted is not None:
+            return self._decrypted
 
         from .crypt import SecurityHandler
 
+        decrypted: PdfObject
+
         obj = self.raw_object
         handler: SecurityHandler = self.handler
         container_ref = obj.container_ref
@@ -1999,7 +2084,12 @@ class DecryptedObjectProxy(PdfObject):
         self._decrypted = decrypted
         return decrypted
 
-    def write_to_stream(self, stream, handler=None, container_ref=None):
+    def write_to_stream(
+        self,
+        stream,
+        handler: Optional['SecurityHandler'] = None,
+        container_ref=None,
+    ):
         # maybe the encryption key for this object changed (due to it being
         # included as part of a larger object or somesuch, without proper
         # dereferencing), so to avoid unexpected shenanigans, let's start from
@@ -2040,9 +2130,10 @@ def pdf_date(dt: datetime) -> TextStringObject:
 
     base_dt = dt.strftime(ASN_DT_FORMAT)
     utc_offset_string = ''
-    if dt.tzinfo is not None:
+    utc_offset = dt.utcoffset()
+    if utc_offset is not None:
         # compute UTC offset string
-        tz_seconds = dt.utcoffset().total_seconds()
+        tz_seconds = utc_offset.total_seconds()
         if not tz_seconds:
             utc_offset_string = 'Z'
         else:
@@ -2063,18 +2154,19 @@ def pdf_date(dt: datetime) -> TextStringObject:
 
 # The year field is the only mandatory one
 MIN_DATE_REGEX = re.compile(r'^D:(\d{4})')
+MIN_DATE_REGEX_LENIENT = re.compile(r'^(?:D:)?(\d{4})')
 TWO_DIGIT_START = re.compile(r'^(\d\d)')
 UTC_OFFSET = re.compile(r"(\d\d)(?:'(\d\d))?'?")
 
 
-def parse_pdf_date(date_str: str) -> datetime:
-    m = MIN_DATE_REGEX.match(date_str)
+def parse_pdf_date(date_str: str, strict: bool = True) -> datetime:
+    m = (MIN_DATE_REGEX if strict else MIN_DATE_REGEX_LENIENT).match(date_str)
     if not m:
         raise PdfReadError(f"{date_str} does not appear to be a date string.")
     year = int(m.group(1))
 
     # now, there are a number of 2-digit groups (anywhere from 0 to 5)
-    date_remaining = date_str[6:]
+    date_remaining = date_str[m.end(0) :]
     lower_order = [1, 1, 0, 0, 0]
 
     for ix in range(5):
