@@ -14,7 +14,7 @@ import os
 import re
 from collections import defaultdict
 from io import BytesIO
-from typing import Generator, Optional, Set, Tuple, Union
+from typing import BinaryIO, Dict, Generator, Optional, Set, Tuple, Union
 
 from . import generic, misc
 from .crypt import (
@@ -38,7 +38,6 @@ from .xref import (
 
 logger = logging.getLogger(__name__)
 
-
 __all__ = [
     'PdfFileReader',
     'HistoricalResolver',
@@ -57,6 +56,7 @@ def parse_catalog_version(version_str) -> Optional[Tuple[int, int]]:
         major = int(m.group(1))
         minor = int(m.group(2))
         return major, minor
+    return None
 
 
 # General remark:
@@ -72,7 +72,7 @@ def read_next_end_line(stream):
             if stream.tell() == 0:
                 raise PdfReadError("Could not read malformed PDF file")
             x = stream.read(1)
-            if stream.tell() < 2:
+            if stream.tell() < 2 or not x:
                 raise PdfReadError("EOL marker not found")
             stream.seek(-2, os.SEEK_CUR)
             if x == b'\n' or x == b'\r':
@@ -84,8 +84,6 @@ def read_next_end_line(stream):
             if x == b'\n' or x == b'\r':  # account for CR+LF
                 stream.seek(-1, os.SEEK_CUR)
                 crlf = True
-            if stream.tell() < 2:
-                raise PdfReadError("EOL marker not found")
             stream.seek(-2, os.SEEK_CUR)
         # if using CR+LF, go back 2 bytes, else 1
         stream.seek(2 if crlf else 1, os.SEEK_CUR)
@@ -133,6 +131,41 @@ def process_data_at_eof(stream) -> int:
     return startxref
 
 
+def _read_header_version(stream: BinaryIO) -> Tuple[int, int]:
+    stream.seek(0)
+    input_version = None
+    header = misc.read_until_whitespace(stream, maxchars=20)
+    # match ignores trailing chars
+    m = header_regex.match(header)
+    if m is not None:
+        major = int(m.group(1))
+        minor = int(m.group(2))
+        input_version = (major, minor)
+    if input_version is None:
+        raise PdfReadError('Illegal PDF header')
+    return input_version
+
+
+def _read_xrefs_and_trailer(
+    stream: BinaryIO, handler_ref: PdfHandler, strict: bool
+) -> Tuple[XRefCache, XRefBuilder]:
+    # start at the end to read the trailer & xref table
+    stream.seek(-1, os.SEEK_END)
+    # This needs to be recorded for incremental update purposes
+    last_startxref = process_data_at_eof(stream)
+
+    # Read the xref table
+    xref_builder = XRefBuilder(
+        handler=handler_ref,
+        stream=stream,
+        strict=strict,
+        last_startxref=last_startxref,
+    )
+    xref_sections = xref_builder.read_xrefs()
+    xref_cache = XRefCache(handler_ref, xref_sections)
+    return xref_cache, xref_builder
+
+
 class PdfFileReader(PdfHandler):
     """Class implementing functionality to read a PDF file and cache
     certain data about it."""
@@ -141,7 +174,7 @@ class PdfFileReader(PdfHandler):
     has_xref_stream = False
     xrefs: XRefCache
 
-    def __init__(self, stream, strict=True):
+    def __init__(self, stream, strict: bool = True):
         """
         Initializes a PdfFileReader object.  This operation can take some time,
         as the PDF stream's cross-reference tables are read into memory.
@@ -152,30 +185,40 @@ class PdfFileReader(PdfHandler):
             problems and also causes some correctable problems to be fatal.
             Defaults to ``True``.
         """
-        self.security_handler: Optional[SecurityHandler] = None
+        self._security_handler: Optional[SecurityHandler] = None
         self.strict = strict
-        self.resolved_objects = {}
+        self.resolved_objects: Dict[Tuple[int, int], generic.PdfObject] = {}
         self._header_version = None
         self._input_version = None
-        self._historical_resolver_cache = {}
+        self._historical_resolver_cache: Dict[int, HistoricalResolver] = {}
         self.stream = stream
-        self.xrefs, self.trailer = self.read()
-        encrypt_dict = self._get_encryption_params()
-        if encrypt_dict is not None:
-            self.security_handler = SecurityHandler.build(encrypt_dict)
+        # first, read the header & PDF version number
+        # (version number can be overridden in the document catalog later)
+        self._header_version = _read_header_version(stream)
+        self.xrefs, xref_builder = _read_xrefs_and_trailer(stream, self, strict)
+        self.last_startxref = xref_builder.last_startxref
+        self.trailer = xref_builder.trailer
+        self.has_xref_stream = xref_builder.has_xref_stream
 
         self._embedded_signatures = None
 
+    @property
+    def security_handler(self):
+        if self.encrypt_dict and not self._security_handler:
+            self._security_handler = SecurityHandler.build(self.encrypt_dict)
+        return self._security_handler
+
     def _xmp_meta_view(self) -> Optional[DocumentMetadata]:
         try:
             from pyhanko.pdf_utils.metadata import xmp_xml
 
             meta_obj = self.root['/Metadata']
         except (ImportError, KeyError):
-            return
+            return None
 
         if isinstance(meta_obj, xmp_xml.MetadataStream):
             return xmp_xml.meta_from_xmp(meta_obj.xmp)
+        return None
 
     @property
     def document_meta_view(self) -> DocumentMetadata:
@@ -188,7 +231,7 @@ class PdfFileReader(PdfHandler):
         except KeyError:
             return DocumentMetadata()
 
-        return view_from_info_dict(info_dict)
+        return view_from_info_dict(info_dict, strict=self.strict)
 
     @property
     def input_version(self):
@@ -199,7 +242,7 @@ class PdfFileReader(PdfHandler):
 
         try:
             version = self.root['/Version']
-            input_version = parse_catalog_version(version)
+            input_version = parse_catalog_version(version) or header_version
         except KeyError:
             input_version = header_version
 
@@ -280,15 +323,25 @@ class PdfFileReader(PdfHandler):
         else:
             return generic.NullObject()
 
-    def _get_encryption_params(self) -> Optional[generic.DictionaryObject]:
+    @property
+    def encrypt_dict(self) -> Optional[generic.DictionaryObject]:
         try:
             encrypt_ref = self.trailer.raw_get('/Encrypt')
         except KeyError:
-            return
+            return None
         if isinstance(encrypt_ref, generic.IndirectObject):
-            return self.get_object(encrypt_ref.reference, never_decrypt=True)
+            encrypt_dict = self.get_object(
+                encrypt_ref.reference, never_decrypt=True
+            )
+        elif not self.strict:
+            encrypt_dict = encrypt_ref
         else:
-            return encrypt_ref
+            raise misc.PdfReadError(
+                "Encryption settings must be an indirect reference"
+            )
+        if not isinstance(encrypt_dict, generic.DictionaryObject):
+            raise misc.PdfReadError("Encryption settings must be a dictionary")
+        return encrypt_dict
 
     @property
     def trailer_view(self) -> generic.DictionaryObject:
@@ -399,7 +452,11 @@ class PdfFileReader(PdfHandler):
         return obj
 
     def _read_object(
-        self, ref, marker, never_decrypt=False, as_metadata_stream=False
+        self,
+        ref: generic.Reference,
+        marker: Union[int, ObjStreamRef, None],
+        never_decrypt: bool = False,
+        as_metadata_stream: bool = False,
     ):
         if marker is None:
             if self.strict:
@@ -456,7 +513,8 @@ class PdfFileReader(PdfHandler):
             and not isinstance(marker, ObjStreamRef)
             and self.encrypted
         ):
-            sh: SecurityHandler = self.security_handler
+            sh: Optional[SecurityHandler] = self.security_handler
+            assert sh is not None
             # make sure the object that lands in the cache is always
             # a proxy object
             retval = generic.proxy_encrypted_obj(retval, sh)
@@ -470,45 +528,6 @@ class PdfFileReader(PdfHandler):
         self.resolved_objects[(generation, idnum)] = obj
         return obj
 
-    def read(self):
-        # first, read the header & PDF version number
-        # (version number can be overridden in the document catalog later)
-        stream = self.stream
-        stream.seek(0)
-        input_version = None
-        try:
-            header = misc.read_until_whitespace(stream, maxchars=20)
-            # match ignores trailing chars
-            m = header_regex.match(header)
-            if m is not None:
-                major = int(m.group(1))
-                minor = int(m.group(2))
-                input_version = (major, minor)
-        except (UnicodeDecodeError, ValueError):
-            pass
-        if input_version is None:
-            raise PdfReadError('Illegal PDF header')
-        self._header_version = input_version
-
-        # start at the end:
-        stream.seek(-1, os.SEEK_END)
-        if not stream.tell():
-            raise PdfReadError('Cannot read an empty file')
-
-        # This needs to be recorded for incremental update purposes
-        self.last_startxref = last_startxref = process_data_at_eof(stream)
-        # Read the xref table
-        xref_builder = XRefBuilder(
-            handler=self,
-            stream=stream,
-            strict=self.strict,
-            last_startxref=last_startxref,
-        )
-        xref_sections = xref_builder.read_xrefs()
-        xref_cache = XRefCache(self, xref_sections)
-        self.has_xref_stream = xref_builder.has_xref_stream
-        return xref_cache, xref_builder.trailer
-
     def decrypt(self, password: Union[str, bytes]) -> AuthResult:
         """
         When using an encrypted PDF file with the standard PDF encryption
@@ -685,7 +704,7 @@ class RawPdfPath:
         self, from_obj, transparent_dereference=True
     ) -> Generator[Tuple[Union[int, str, None], generic.PdfObject], None, None]:
         current_obj = from_obj
-        elem = None
+        elem: Union[int, str, None] = None
         for ix, entry in enumerate(self.path):
             if not transparent_dereference:
                 yield elem, current_obj
@@ -807,7 +826,7 @@ class HistoricalResolver(PdfHandler):
         except KeyError:
             return DocumentMetadata()
 
-        return view_from_info_dict(info_dict)
+        return view_from_info_dict(info_dict, strict=self.reader.strict)
 
     @property
     def document_id(self) -> Tuple[bytes, bytes]:
@@ -815,11 +834,14 @@ class HistoricalResolver(PdfHandler):
         return id_arr[0].original_bytes, id_arr[1].original_bytes
 
     def __init__(self, reader: PdfFileReader, revision):
-        self.cache = {}
+        self.cache: Dict[generic.Reference, generic.PdfObject] = {}
         self.reader = reader
         self.revision = revision
         self._trailer = self.reader.trailer.flatten(self.revision)
-        self._indirect_object_access_cache = None
+        self._trailer.container_ref = generic.TrailerReference(self)
+        self._indirect_object_access_cache: Optional[
+            Dict[generic.Reference, Set[RawPdfPath]]
+        ] = None
 
     @property
     def trailer_view(self) -> generic.DictionaryObject:
@@ -966,7 +988,9 @@ class HistoricalResolver(PdfHandler):
         # as long as the ID is strictly less than meta.size
         return ref.idnum >= meta.size
 
-    def collect_dependencies(self, obj: generic.PdfObject, since_revision=None):
+    def collect_dependencies(
+        self, obj: generic.PdfObject, since_revision=None
+    ) -> Set[generic.Reference]:
         """
         Collect all indirect references used by an object and its descendants.
 
@@ -983,7 +1007,7 @@ class HistoricalResolver(PdfHandler):
         :return:
             A :class:`set` of :class:`~.generic.Reference` objects.
         """
-        result_set = set()
+        result_set: Set[generic.Reference] = set()
         self._collect_indirect_references(obj, result_set, since_revision)
         return result_set
 
@@ -1030,8 +1054,8 @@ class HistoricalResolver(PdfHandler):
         # We flatten everything when we're done
         def _compute_paths_to_refs(
             obj,
-            cur_path: misc.ConsList,
-            seen_in_path: misc.ConsList,
+            cur_path: misc.ConsList[Union[str, int]],
+            seen_in_path: misc.ConsList[generic.Reference],
             *,
             is_page_tree,
             page_tree_objs,
