@@ -35,15 +35,30 @@ from .rules_api import QualifiedWhitelistRule, ReferenceUpdate, WhitelistRule
 # export list for compatibility with the old module
 
 __all__ = [
-    'ModificationLevel', 'SuspiciousModification',
-    'QualifiedWhitelistRule', 'WhitelistRule', 'qualify', 'ReferenceUpdate',
-    'DocInfoRule', 'DSSCompareRule', 'MetadataUpdateRule',
-    'CatalogModificationRule', 'ObjectStreamRule', 'XrefStreamRule',
-    'FormUpdatingRule', 'FormUpdate',
-    'FieldMDPRule', 'FieldComparisonSpec', 'FieldComparisonContext',
-    'GenericFieldModificationRule', 'SigFieldCreationRule',
-    'SigFieldModificationRule', 'BaseFieldModificationRule',
-    'DiffPolicy', 'StandardDiffPolicy',
-    'DEFAULT_DIFF_POLICY', 'NO_CHANGES_DIFF_POLICY',
-    'DiffResult'
+    'ModificationLevel',
+    'SuspiciousModification',
+    'QualifiedWhitelistRule',
+    'WhitelistRule',
+    'qualify',
+    'ReferenceUpdate',
+    'DocInfoRule',
+    'DSSCompareRule',
+    'MetadataUpdateRule',
+    'CatalogModificationRule',
+    'ObjectStreamRule',
+    'XrefStreamRule',
+    'FormUpdatingRule',
+    'FormUpdate',
+    'FieldMDPRule',
+    'FieldComparisonSpec',
+    'FieldComparisonContext',
+    'GenericFieldModificationRule',
+    'SigFieldCreationRule',
+    'SigFieldModificationRule',
+    'BaseFieldModificationRule',
+    'DiffPolicy',
+    'StandardDiffPolicy',
+    'DEFAULT_DIFF_POLICY',
+    'NO_CHANGES_DIFF_POLICY',
+    'DiffResult',
 ]
