@@ -9,19 +9,16 @@ CMS is defined in :rfc:`5652`. To parse CMS messages, pyHanko relies heavily on
 import hashlib
 import logging
 from dataclasses import dataclass
-from typing import IO, Iterable, List, Optional, Tuple, Union
+from typing import IO, Iterable, List, Tuple, Union
 
-from asn1crypto import algos, cms, keys, pem, tsp, x509
+from asn1crypto import algos, cms, tsp, x509
 from asn1crypto.algos import SignedDigestAlgorithm
 from cryptography.hazmat.primitives import hashes, serialization
 from cryptography.hazmat.primitives.asymmetric import padding
 from cryptography.hazmat.primitives.asymmetric.rsa import RSAPublicKey
 from cryptography.hazmat.primitives.asymmetric.utils import Prehashed
-from pyhanko_certvalidator.registry import (
-    CertificateStore,
-    SimpleCertificateStore,
-)
 
+from pyhanko.keys import *
 from pyhanko.pdf_utils import misc
 
 __all__ = [
@@ -30,20 +27,14 @@ __all__ = [
     'find_unique_cms_attribute',
     'NonexistentAttributeError',
     'MultivaluedAttributeError',
-    'CertificateStore',
-    'SimpleCertificateStore',
     'SigningError',
     'UnacceptableSignerError',
     'SignedDataCerts',
     'extract_signer_info',
     'extract_certificate_info',
-    'load_certs_from_pemder',
-    'load_cert_from_pemder',
-    'load_private_key_from_pemder',
-    'load_certs_from_pemder_data',
-    'load_private_key_from_pemder_data',
     'get_cms_hash_algo_for_mechanism',
     'get_pyca_cryptography_hash',
+    'get_pyca_cryptography_hash_for_signing',
     'optimal_pss_params',
     'process_pss_params',
     'as_signing_certificate',
@@ -54,6 +45,12 @@ __all__ = [
     'byte_range_digest',
     'ValueErrorWithMessage',
     'CMSStructuralError',
+    # reexported for compatibility
+    'load_cert_from_pemder',
+    'load_certs_from_pemder',
+    'load_certs_from_pemder_data',
+    'load_private_key_from_pemder',
+    'load_private_key_from_pemder_data',
 ]
 
 logger = logging.getLogger(__name__)
@@ -68,6 +65,7 @@ class ValueErrorWithMessage(ValueError):
 
     def __init__(self, failure_message):
         self.failure_message = str(failure_message)
+        super().__init__(failure_message)
 
 
 class CMSStructuralError(ValueErrorWithMessage):
@@ -349,14 +347,18 @@ class UnacceptableSignerError(SigningError):
     pass
 
 
-def get_pyca_cryptography_hash(
-    algorithm, prehashed=False
-) -> Union[hashes.HashAlgorithm, Prehashed]:
+def get_pyca_cryptography_hash(algorithm) -> Union[hashes.HashAlgorithm]:
     if algorithm.lower() == 'shake256':
         # force the output length to 64 bytes = 512 bits
-        hash_algo = hashes.SHAKE256(digest_size=64)
+        return hashes.SHAKE256(digest_size=64)
     else:
-        hash_algo = getattr(hashes, algorithm.upper())()
+        return getattr(hashes, algorithm.upper())()
+
+
+def get_pyca_cryptography_hash_for_signing(
+    algorithm, prehashed=False
+) -> Union[hashes.HashAlgorithm, Prehashed]:
+    hash_algo = get_pyca_cryptography_hash(algorithm)
     return Prehashed(hash_algo) if prehashed else hash_algo
 
 
@@ -414,8 +416,8 @@ def process_pss_params(
         )
     salt_len: int = params['salt_length'].native
 
-    mgf_md = get_pyca_cryptography_hash(mgf_md_name, prehashed=False)
-    md = get_pyca_cryptography_hash(md_name, prehashed=prehashed)
+    mgf_md = get_pyca_cryptography_hash(mgf_md_name)
+    md = get_pyca_cryptography_hash_for_signing(md_name, prehashed=prehashed)
     pss_padding = padding.PSS(
         mgf=padding.MGF1(algorithm=mgf_md), salt_length=salt_len
     )
@@ -439,9 +441,9 @@ def optimal_pss_params(
 
     digest_algorithm = digest_algorithm.lower()
 
-    key: RSAPublicKey = serialization.load_der_public_key(
-        cert.public_key.dump()
-    )
+    key = serialization.load_der_public_key(cert.public_key.dump())
+    if not isinstance(key, RSAPublicKey):
+        raise SigningError(f"Expected RSA key, but got key of type {type(key)}")
     md = get_pyca_cryptography_hash(digest_algorithm)
     # the PSS salt calculation function is not in the .pyi file, apparently.
     # noinspection PyUnresolvedReferences
@@ -464,132 +466,6 @@ def optimal_pss_params(
     )
 
 
-def load_certs_from_pemder(cert_files):
-    """
-    A convenience function to load PEM/DER-encoded certificates from files.
-
-    :param cert_files:
-        An iterable of file names.
-    :return:
-        A generator producing :class:`.asn1crypto.x509.Certificate` objects.
-    """
-    for cert_file in cert_files:
-        with open(cert_file, 'rb') as f:
-            cert_data_bytes = f.read()
-        yield from load_certs_from_pemder_data(cert_data_bytes)
-
-
-def load_certs_from_pemder_data(cert_data_bytes: bytes):
-    """
-    A convenience function to load PEM/DER-encoded certificates from
-    binary data.
-
-    :param cert_data_bytes:
-        ``bytes`` object from which to extract certificates.
-    :return:
-        A generator producing :class:`.asn1crypto.x509.Certificate` objects.
-    """
-    # use the pattern from the asn1crypto docs
-    # to distinguish PEM/DER and read multiple certs
-    # from one PEM file (if necessary)
-    if pem.detect(cert_data_bytes):
-        pems = pem.unarmor(cert_data_bytes, multiple=True)
-        for type_name, _, der in pems:
-            if type_name is None or type_name.lower() == 'certificate':
-                yield x509.Certificate.load(der)
-            else:  # pragma: nocover
-                logger.debug(
-                    f'Skipping PEM block of type {type_name} in '
-                    f'CA chain file.'
-                )
-    else:
-        # no need to unarmor, just try to load it immediately
-        yield x509.Certificate.load(cert_data_bytes)
-
-
-def load_cert_from_pemder(cert_file):
-    """
-    A convenience function to load a single PEM/DER-encoded certificate
-    from a file.
-
-    :param cert_file:
-        A file name.
-    :return:
-        An :class:`.asn1crypto.x509.Certificate` object.
-    """
-    certs = list(load_certs_from_pemder([cert_file]))
-    if len(certs) != 1:
-        raise ValueError(f"Number of certs in {cert_file} should be exactly 1")
-    return certs[0]
-
-
-def load_private_key_from_pemder(
-    key_file, passphrase: Optional[bytes]
-) -> keys.PrivateKeyInfo:
-    """
-    A convenience function to load PEM/DER-encoded keys from files.
-
-    :param key_file:
-        File to read the key from.
-    :param passphrase:
-        Key passphrase.
-    :return:
-        A private key encoded as an unencrypted PKCS#8 PrivateKeyInfo object.
-    """
-    with open(key_file, 'rb') as f:
-        key_bytes = f.read()
-    return load_private_key_from_pemder_data(key_bytes, passphrase=passphrase)
-
-
-def load_private_key_from_pemder_data(
-    key_bytes: bytes, passphrase: Optional[bytes]
-) -> keys.PrivateKeyInfo:
-    """
-    A convenience function to load PEM/DER-encoded keys from binary data.
-
-    :param key_bytes:
-        ``bytes`` object to read the key from.
-    :param passphrase:
-        Key passphrase.
-    :return:
-        A private key encoded as an unencrypted PKCS#8 PrivateKeyInfo object.
-    """
-    load_fun = (
-        serialization.load_pem_private_key
-        if pem.detect(key_bytes)
-        else serialization.load_der_private_key
-    )
-    return _translate_pyca_cryptography_key_to_asn1(
-        load_fun(key_bytes, password=passphrase)
-    )
-
-
-def _translate_pyca_cryptography_key_to_asn1(
-    private_key,
-) -> keys.PrivateKeyInfo:
-    # Store the cert and key as generic ASN.1 structures for more
-    # "standardised" introspection. This comes at the cost of some encoding/
-    # decoding operations, but those should be fairly insignificant in the
-    # grand scheme of things.
-    #
-    # Note: we're not losing any memory protections here:
-    #  (https://cryptography.io/en/latest/limitations.html)
-    # Arguably, memory safety is nigh impossible to obtain in a Python
-    # context anyhow, and people with that kind of Serious (TM) security
-    # requirements should be using HSMs to manage keys.
-    return keys.PrivateKeyInfo.load(
-        private_key.private_bytes(
-            serialization.Encoding.DER,
-            serialization.PrivateFormat.PKCS8,
-            serialization.NoEncryption(),
-        )
-    )
-
-
-def _translate_pyca_cryptography_cert_to_asn1(cert) -> x509.Certificate:
-    return x509.Certificate.load(cert.public_bytes(serialization.Encoding.DER))
-
-
 @dataclass(frozen=True)
 class SignedDataCerts:
     """
