@@ -4,8 +4,9 @@ from typing import Optional
 
 from asn1crypto import cms, tsp
 from cryptography.hazmat.primitives import hashes
+from pyhanko_certvalidator.registry import CertificateStore
 
-from ..general import CertificateStore, get_pyca_cryptography_hash
+from ..general import get_pyca_cryptography_hash
 
 __all__ = [
     'TimestampRequestError',
@@ -74,7 +75,7 @@ def handle_tsp_response(
     if nonce is not None and nonce_received != nonce:
         raise TimestampRequestError(
             f'Time stamping authority sent back bad nonce value. Expected '
-            f'{nonce}, but got {nonce_received}.'
+            f'{nonce.hex()}, but got {hex(nonce_received)}.'
         )
     return tst
 
