@@ -1,17 +1,15 @@
 from datetime import datetime
+from typing import Optional
 
 import tzlocal
 from asn1crypto import algos, cms, core, keys, tsp, x509
 from cryptography.hazmat.primitives import hashes, serialization
 from cryptography.hazmat.primitives.asymmetric.padding import PKCS1v15
 from cryptography.hazmat.primitives.asymmetric.rsa import RSAPrivateKey
+from pyhanko_certvalidator.registry import CertificateStore
 
 from .. import general
-from ..general import (
-    CertificateStore,
-    get_pyca_cryptography_hash,
-    simple_cms_attribute,
-)
+from ..general import get_pyca_cryptography_hash, simple_cms_attribute
 from .api import TimeStamper
 from .common_utils import get_nonce
 
@@ -29,8 +27,8 @@ class DummyTimeStamper(TimeStamper):
         self,
         tsa_cert: x509.Certificate,
         tsa_key: keys.PrivateKeyInfo,
-        certs_to_embed: CertificateStore = None,
-        fixed_dt: datetime = None,
+        certs_to_embed: Optional[CertificateStore] = None,
+        fixed_dt: Optional[datetime] = None,
         include_nonce=True,
         override_md=None,
     ):
@@ -56,7 +54,7 @@ class DummyTimeStamper(TimeStamper):
             {'algorithm': md_algorithm}
         )
         dt = self.fixed_dt or datetime.now(tz=tzlocal.get_localzone())
-        tst_info = {
+        tst_info_args = {
             'version': 'v1',
             # See http://oidref.com/1.3.6.1.4.1.4146.2.2
             # I don't really care too much, this is a testing device anyway
@@ -70,9 +68,9 @@ class DummyTimeStamper(TimeStamper):
             ),
         }
         if req['nonce'].native is not None:
-            tst_info['nonce'] = req['nonce']
+            tst_info_args['nonce'] = req['nonce']
 
-        tst_info = tsp.TSTInfo(tst_info)
+        tst_info = tsp.TSTInfo(tst_info_args)
         tst_info_data = tst_info.dump()
         md_spec = get_pyca_cryptography_hash(md_algorithm)
         md = hashes.Hash(md_spec)
