@@ -2,10 +2,11 @@ import hashlib
 import logging
 from dataclasses import dataclass
 from dataclasses import field as data_field
-from typing import Optional
+from typing import Iterable, Optional
 
 from asn1crypto import crl as asn1_crl
 from asn1crypto import ocsp as asn1_ocsp
+from asn1crypto import x509
 from asn1crypto.x509 import Certificate
 from pyhanko_certvalidator import CertificateValidator, ValidationContext
 from pyhanko_certvalidator.path import ValidationPath
@@ -26,6 +27,7 @@ __all__ = [
     'DocumentSecurityStore',
     'async_add_validation_info',
     'collect_validation_info',
+    'enumerate_ocsp_certs',
 ]
 
 from ...pdf_utils.crypt import SerialisedCredential
@@ -95,7 +97,7 @@ class DocumentSecurityStore:
 
     def __init__(
         self,
-        writer: BasePdfFileWriter,
+        writer: Optional[BasePdfFileWriter],
         certs=None,
         ocsps=None,
         crls=None,
@@ -260,7 +262,7 @@ class DocumentSecurityStore:
 
         return pdf_dict
 
-    def load_certs(self):
+    def load_certs(self) -> Iterable[x509.Certificate]:
         """
         Return a generator that parses and yields all certificates in the DSS.
 
@@ -380,7 +382,7 @@ class DocumentSecurityStore:
         crls=None,
         paths=None,
         validation_context=None,
-        embed_roots: bool = True
+        embed_roots: bool = True,
     ) -> 'DocumentSecurityStore':
         """
         Add or update a DSS, and optionally associate the new information with a
@@ -441,7 +443,7 @@ class DocumentSecurityStore:
         else:
             identifier = None
 
-        def _certs():
+        def _certs() -> Iterable[x509.Certificate]:
             yield from certs or ()
             path: ValidationPath
             for path in paths or ():
@@ -487,7 +489,7 @@ class DocumentSecurityStore:
         validation_context=None,
         force_write: bool = False,
         embed_roots: bool = True,
-        file_credential: Optional[SerialisedCredential] = None
+        file_credential: Optional[SerialisedCredential] = None,
     ):
         """
         Wrapper around :meth:`supply_dss_in_writer`.
