@@ -1,7 +1,22 @@
 import asyncio
 import logging
+from dataclasses import dataclass
 from datetime import datetime
-from typing import IO, Iterable, List, Optional, Tuple, Type, TypeVar, Union
+from typing import (
+    IO,
+    Any,
+    Awaitable,
+    Dict,
+    Generic,
+    Iterable,
+    List,
+    Optional,
+    Tuple,
+    Type,
+    TypeVar,
+    Union,
+    overload,
+)
 
 from asn1crypto import cms, core, tsp, x509
 from cryptography.exceptions import InvalidSignature
@@ -18,14 +33,12 @@ from pyhanko_certvalidator.errors import (
     PathBuildingError,
     PathValidationError,
     RevokedError,
+    StaleRevinfoError,
     ValidationError,
 )
 from pyhanko_certvalidator.ltv.errors import TimeSlideFailure
 from pyhanko_certvalidator.path import ValidationPath
-from pyhanko_certvalidator.policy_decl import (
-    AlgorithmUsagePolicy,
-    PKIXValidationParams,
-)
+from pyhanko_certvalidator.policy_decl import PKIXValidationParams
 from pyhanko_certvalidator.validate import ACValidationResult, async_validate_ac
 
 from pyhanko.sign.general import (
@@ -42,6 +55,7 @@ from pyhanko.sign.general import (
 )
 
 from ...pdf_utils import misc
+from ...pdf_utils.misc import lift_iterable_async
 from ..ades.report import AdESFailure, AdESIndeterminate
 from . import errors
 from .settings import KeyUsageConstraints
@@ -56,6 +70,7 @@ from .status import (
 )
 from .utils import (
     DEFAULT_ALGORITHM_USAGE_POLICY,
+    CMSAlgorithmUsagePolicy,
     extract_message_digest,
     validate_raw,
 )
@@ -73,15 +88,32 @@ __all__ = [
     'extract_certs_for_validation',
     'collect_signer_attr_status',
     'validate_algorithm_protection',
+    'get_signing_cert_attr',
 ]
 
-from ...pdf_utils.misc import lift_iterable_async
-
 logger = logging.getLogger(__name__)
 
 StatusType = TypeVar('StatusType', bound=SignatureStatus)
 
 
+def get_signing_cert_attr(
+    signed_attrs: cms.CMSAttributes,
+) -> Union[tsp.SigningCertificate, tsp.SigningCertificateV2, None]:
+    """
+    Retrieve the ``signingCertificate`` or ``signingCertificateV2`` attribute
+    (giving preference to the latter) from a signature's signed attributes.
+
+    :param signed_attrs:
+        Signed attributes.
+    :return:
+        The value of the attribute, if present, else ``None``.
+    """
+    attr = _grab_signing_cert_attr(signed_attrs, v2=True)
+    if attr is None:
+        attr = _grab_signing_cert_attr(signed_attrs, v2=False)
+    return attr
+
+
 def _grab_signing_cert_attr(signed_attrs, v2: bool):
     # TODO check certificate policies, enforce restrictions on chain of trust
     # TODO document and/or mark as internal API explicitly
@@ -110,12 +142,9 @@ def _check_signing_certificate(
     # TODO check certificate policies, enforce restrictions on chain of trust
     # TODO document and/or mark as internal API explicitly
 
-    attr = _grab_signing_cert_attr(signed_attrs, v2=True)
+    attr = get_signing_cert_attr(signed_attrs)
     if attr is None:
-        attr = _grab_signing_cert_attr(signed_attrs, v2=False)
-
-    if attr is None:
-        # if neither attr is present -> no constraints
+        # if not present -> no constraints
         return
 
     # For the main signer cert, we only care about the first value, the others
@@ -191,7 +220,7 @@ def validate_sig_integrity(
     cert: x509.Certificate,
     expected_content_type: str,
     actual_digest: bytes,
-    algorithm_usage_policy: Optional[AlgorithmUsagePolicy] = None,
+    algorithm_usage_policy: Optional[CMSAlgorithmUsagePolicy] = None,
     time_indic: Optional[datetime] = None,
 ) -> Tuple[bool, bool]:
     """
@@ -231,19 +260,34 @@ def validate_sig_integrity(
     digest_algorithm_obj = signer_info['digest_algorithm']
     md_algorithm = digest_algorithm_obj['algorithm'].native
     if algorithm_usage_policy is not None:
-        algo_allowed = algorithm_usage_policy.signature_algorithm_allowed(
+        sig_algo_allowed = algorithm_usage_policy.signature_algorithm_allowed(
             signature_algorithm, moment=time_indic, public_key=cert.public_key
         )
-        if not algo_allowed:
+        if not sig_algo_allowed:
             msg = (
                 f"The algorithm {signature_algorithm['algorithm'].native} "
                 f"is not allowed by the current usage policy."
             )
-            if algo_allowed.failure_reason is not None:
-                msg += f" Reason: {algo_allowed.failure_reason}."
+            if sig_algo_allowed.failure_reason is not None:
+                msg += f" Reason: {sig_algo_allowed.failure_reason}."
             raise errors.DisallowedAlgorithmError(
-                msg, permanent=algo_allowed.not_allowed_after is None
+                msg, permanent=sig_algo_allowed.not_allowed_after is None
             )
+        digest_algo_allowed = algorithm_usage_policy.digest_algorithm_allowed(
+            digest_algorithm_obj,
+            moment=time_indic,
+        )
+        if not digest_algo_allowed:
+            msg = (
+                f"The algorithm {digest_algorithm_obj['algorithm'].native} "
+                f"is not allowed by the current usage policy."
+            )
+            if digest_algo_allowed.failure_reason is not None:
+                msg += f" Reason: {digest_algo_allowed.failure_reason}."
+            raise errors.DisallowedAlgorithmError(
+                msg, permanent=digest_algo_allowed.not_allowed_after is None
+            )
+
     signature = signer_info['signature'].native
 
     signed_attrs_orig: cms.CMSAttributes = signer_info['signed_attrs']
@@ -371,14 +415,15 @@ def extract_certs_for_validation(
 
 async def cms_basic_validation(
     signed_data: cms.SignedData,
-    raw_digest: bytes = None,
+    raw_digest: Optional[bytes] = None,
     validation_context: Optional[ValidationContext] = None,
-    status_kwargs: dict = None,
+    status_kwargs: Optional[dict] = None,
     validation_path: Optional[ValidationPath] = None,
     pkix_validation_params: Optional[PKIXValidationParams] = None,
+    algorithm_policy: Optional[CMSAlgorithmUsagePolicy] = None,
     *,
     key_usage_settings: KeyUsageConstraints,
-):
+) -> Dict[str, Any]:
     """
     Perform basic validation of CMS and PKCS#7 signatures in isolation
     (i.e. integrity and trust checks).
@@ -390,10 +435,14 @@ async def cms_basic_validation(
     cert = cert_info.signer_cert
     other_certs = cert_info.other_certs
 
-    algorithm_policy = None
     time_indic = None
     if validation_context is not None:
-        algorithm_policy = validation_context.algorithm_policy
+        algorithm_policy = (
+            algorithm_policy
+            or CMSAlgorithmUsagePolicy.lift_policy(
+                validation_context.algorithm_policy
+            )
+        )
         time_indic = validation_context.best_signature_time
     validation_context = validation_context or ValidationContext()
     if algorithm_policy is None:
@@ -413,6 +462,12 @@ async def cms_basic_validation(
         md = hashes.Hash(md_spec)
         md.update(raw)
         raw_digest = md.finalize()
+    elif eci['content'] is not core.VOID:
+        raise errors.SignatureValidationError(
+            "CMS structural error: detached signatures should not have "
+            "encapsulated data",
+            ades_subindication=AdESFailure.FORMAT_FAILURE,
+        )
 
     # first, do the cryptographic identity checks
     # TODO theoretically (e.g. DSA with param inheritance) this requires
@@ -434,13 +489,14 @@ async def cms_basic_validation(
         ) from e
 
     # next, validate trust
-    ades_status = path = revo_details = None
+    ades_status = path = revo_details = error_time_horizon = None
     if valid:
         try:
             validation_context.certificate_registry.register_multiple(
                 other_certs
             )
 
+            paths: CancelableAsyncIterator[ValidationPath]
             if validation_path is not None:
                 paths = lift_iterable_async([validation_path])
             else:
@@ -448,18 +504,25 @@ async def cms_basic_validation(
                     cert
                 )
 
-            ades_status, revo_details, path = await validate_cert_usage(
+            op_result = await validate_cert_usage(
                 cert,
                 validation_context,
                 key_usage_settings=key_usage_settings,
                 paths=paths,
                 pkix_validation_params=pkix_validation_params,
             )
+            ades_status = op_result.error_subindic
+            revo_details = op_result.revo_details
+            path = op_result.success_result or op_result.error_path
+            error_time_horizon = op_result.error_time_horizon
         except ValueError as e:
             logger.error("Processing error in validation process", exc_info=e)
             ades_status = AdESIndeterminate.CERTIFICATE_CHAIN_GENERAL_FAILURE
 
     status_kwargs = status_kwargs or {}
+    status_kwargs['validation_time'] = (
+        None if validation_context is None else validation_context.moment
+    )
     status_kwargs.update(
         intact=intact,
         valid=valid,
@@ -469,6 +532,7 @@ async def cms_basic_validation(
         trust_problem_indic=ades_status,
         validation_path=path,
         revocation_details=revo_details,
+        error_time_horizon=error_time_horizon,
     )
     return status_kwargs
 
@@ -479,13 +543,14 @@ async def validate_cert_usage(
     key_usage_settings: KeyUsageConstraints,
     paths: CancelableAsyncIterator[ValidationPath],
     pkix_validation_params: Optional[PKIXValidationParams] = None,
-):
+) -> 'CertvalidatorOperationResult[ValidationPath]':
     """
     Low-level certificate validation routine.
     Internal API.
     """
 
-    async def _check():
+    async def _check() -> ValidationPath:
+        # validate usage without going through pyhanko_certvalidator
         key_usage_settings.validate(cert)
         return await find_valid_path(
             cert,
@@ -494,24 +559,43 @@ async def validate_cert_usage(
             pkix_validation_params=pkix_validation_params,
         )
 
-    # validate usage without going through pyhanko_certvalidator
-    ades_status, revo_details, path = await handle_certvalidator_errors(
-        _check()
-    )
-    if ades_status is not None:
-        subj = cert.subject.human_friendly
-        logger.warning(f"Chain of trust validation for {subj} failed.")
-    return ades_status, revo_details, path
+    return await handle_certvalidator_errors(_check())
+
+
+@overload
+async def async_validate_cms_signature(
+    signed_data: cms.SignedData,
+    *,
+    status_cls: Type[StatusType],
+    raw_digest: Optional[bytes] = None,
+    validation_context: Optional[ValidationContext] = None,
+    status_kwargs: Optional[dict] = None,
+    key_usage_settings: Optional[KeyUsageConstraints] = None,
+) -> StatusType:
+    ...
+
+
+@overload
+async def async_validate_cms_signature(
+    signed_data: cms.SignedData,
+    *,
+    raw_digest: Optional[bytes] = None,
+    validation_context: Optional[ValidationContext] = None,
+    status_kwargs: Optional[dict] = None,
+    key_usage_settings: Optional[KeyUsageConstraints] = None,
+) -> SignatureStatus:
+    ...
 
 
 async def async_validate_cms_signature(
     signed_data: cms.SignedData,
-    status_cls: Type[StatusType] = SignatureStatus,
-    raw_digest: bytes = None,
-    validation_context: ValidationContext = None,
-    status_kwargs: dict = None,
-    key_usage_settings: KeyUsageConstraints = None,
-):
+    status_cls=SignatureStatus,
+    raw_digest: Optional[bytes] = None,
+    validation_context: Optional[ValidationContext] = None,
+    status_kwargs: Optional[dict] = None,
+    key_usage_settings: Optional[KeyUsageConstraints] = None,
+    algorithm_policy: Optional[CMSAlgorithmUsagePolicy] = None,
+) -> StatusType:
     """
     Validate a CMS signature (i.e. a ``SignedData`` object).
 
@@ -529,10 +613,20 @@ async def async_validate_cms_signature(
     :param key_usage_settings:
         A :class:`.KeyUsageConstraints` object specifying which key usages
         must or must not be present in the signer's certificate.
+    :param algorithm_policy:
+        The algorithm usage policy for the signature validation.
+
+        .. warning::
+            This is distinct from the algorithm usage policy used for
+            certificate validation, but the latter will be used as a fallback
+            if this parameter is not specified.
+
+            It is nonetheless recommended to align both policies unless
+            there is a clear reason to do otherwise.
     :return:
         A :class:`.SignatureStatus` object (or an instance of a proper subclass)
     """
-    key_usage_settings = status_cls.default_usage_constraints(
+    eff_key_usage_settings = status_cls.default_usage_constraints(
         key_usage_settings
     )
     status_kwargs = await cms_basic_validation(
@@ -540,8 +634,10 @@ async def async_validate_cms_signature(
         raw_digest,
         validation_context,
         status_kwargs,
-        key_usage_settings=key_usage_settings,
+        key_usage_settings=eff_key_usage_settings,
+        algorithm_policy=algorithm_policy,
     )
+    # noinspection PyArgumentList
     return status_cls(**status_kwargs)
 
 
@@ -562,7 +658,7 @@ def extract_self_reported_ts(signer_info: cms.SignerInfo) -> Optional[datetime]:
         st = find_unique_cms_attribute(sa, 'signing_time')
         return st.native
     except (NonexistentAttributeError, MultivaluedAttributeError):
-        pass
+        return None
 
 
 def extract_tst_data(
@@ -592,7 +688,7 @@ def extract_tst_data(
         tst_signed_data = tst['content']
         return tst_signed_data
     except (NonexistentAttributeError, MultivaluedAttributeError):
-        pass
+        return None
 
 
 def compute_signature_tst_digest(
@@ -630,7 +726,7 @@ def compute_signature_tst_digest(
 
 async def collect_timing_info(
     signer_info: cms.SignerInfo,
-    ts_validation_context: ValidationContext,
+    ts_validation_context: Optional[ValidationContext],
     raw_digest: bytes,
 ):
     """
@@ -647,7 +743,7 @@ async def collect_timing_info(
         validation of the content timestamp token, if there is one)
     """
 
-    status_kwargs = {}
+    status_kwargs: Dict[str, Any] = {}
 
     # timestamp-related validation
     signer_reported_dt = extract_self_reported_ts(signer_info)
@@ -656,10 +752,12 @@ async def collect_timing_info(
 
     tst_signed_data = extract_tst_data(signer_info, signed=False)
     if tst_signed_data is not None:
+        tst_signature_digest = compute_signature_tst_digest(signer_info)
+        assert tst_signature_digest is not None
         tst_validity_kwargs = await validate_tst_signed_data(
             tst_signed_data,
             ts_validation_context,
-            compute_signature_tst_digest(signer_info),
+            tst_signature_digest,
         )
         tst_validity = TimestampSignatureStatus(**tst_validity_kwargs)
         status_kwargs['timestamp_validity'] = tst_validity
@@ -683,6 +781,7 @@ async def validate_tst_signed_data(
     tst_signed_data: cms.SignedData,
     validation_context: Optional[ValidationContext],
     expected_tst_imprint: bytes,
+    algorithm_policy: Optional[CMSAlgorithmUsagePolicy] = None,
 ):
     """
     Validate the ``SignedData`` of a time stamp token.
@@ -695,6 +794,16 @@ async def validate_tst_signed_data(
     :param expected_tst_imprint:
         The expected message imprint value that should be contained in
         the encapsulated ``TSTInfo``.
+    :param algorithm_policy:
+        The algorithm usage policy for the signature validation.
+
+        .. warning::
+            This is distinct from the algorithm usage policy used for
+            certificate validation, but the latter will be used as a fallback
+            if this parameter is not specified.
+
+            It is nonetheless recommended to align both policies unless
+            there is a clear reason to do otherwise.
     :return:
         Keyword arguments for a :class:`.TimeStampSignatureStatus`.
     """
@@ -716,6 +825,7 @@ async def validate_tst_signed_data(
         validation_context=validation_context,
         status_kwargs={'timestamp': timestamp},
         key_usage_settings=ku_settings,
+        algorithm_policy=algorithm_policy,
     )
     # compare the expected TST digest against the message imprint
     # inside the signed data
@@ -733,7 +843,12 @@ async def process_certified_attrs(
     acs: Iterable[cms.AttributeCertificateV2],
     signer_cert: x509.Certificate,
     validation_context: ValidationContext,
-) -> Tuple[List[ACValidationResult], List[Exception]]:
+) -> Tuple[
+    List[ACValidationResult],
+    List[
+        Union[PathValidationError, PathBuildingError, InvalidCertificateError]
+    ],
+]:
     jobs = [
         async_validate_ac(ac, validation_context, holder_cert=signer_cert)
         for ac in acs
@@ -743,7 +858,11 @@ async def process_certified_attrs(
     for job in asyncio.as_completed(jobs):
         try:
             results.append(await job)
-        except (PathBuildingError, PathValidationError) as e:
+        except (
+            PathBuildingError,
+            PathValidationError,
+            InvalidCertificateError,
+        ) as e:
             errors.append(e)
     return results, errors
 
@@ -767,7 +886,7 @@ async def collect_signer_attr_status(
             str(e), ades_subindication=AdESFailure.FORMAT_FAILURE
         ) from e
 
-    result = {}
+    result: Dict[str, Any] = {}
     cades_ac_results = None
     cades_ac_errors = None
     if signer_attrs is not None:
@@ -852,10 +971,11 @@ async def collect_signer_attr_status(
 async def async_validate_detached_cms(
     input_data: Union[bytes, IO, cms.ContentInfo, cms.EncapsulatedContentInfo],
     signed_data: cms.SignedData,
-    signer_validation_context: ValidationContext = None,
-    ts_validation_context: ValidationContext = None,
-    ac_validation_context: ValidationContext = None,
-    key_usage_settings: KeyUsageConstraints = None,
+    signer_validation_context: Optional[ValidationContext] = None,
+    ts_validation_context: Optional[ValidationContext] = None,
+    ac_validation_context: Optional[ValidationContext] = None,
+    key_usage_settings: Optional[KeyUsageConstraints] = None,
+    algorithm_policy: Optional[CMSAlgorithmUsagePolicy] = None,
     chunk_size=misc.DEFAULT_CHUNK_SIZE,
     max_read=None,
 ) -> StandardCMSSignatureStatus:
@@ -889,6 +1009,16 @@ async def async_validate_detached_cms(
         .. note::
             :rfc:`5755` requires attribute authority trust roots to be specified
             explicitly; hence why there's no default.
+    :param algorithm_policy:
+        The algorithm usage policy for the signature validation.
+
+        .. warning::
+            This is distinct from the algorithm usage policy used for
+            certificate validation, but the latter will be used as a fallback
+            if this parameter is not specified.
+
+            It is nonetheless recommended to align both policies unless
+            there is a clear reason to do otherwise.
     :param key_usage_settings:
         Key usage parameters for the signer.
     :param chunk_size:
@@ -927,6 +1057,7 @@ async def async_validate_detached_cms(
         validation_context=signer_validation_context,
         status_kwargs=status_kwargs,
         key_usage_settings=key_usage_settings,
+        algorithm_policy=algorithm_policy,
     )
     cert_info = extract_certificate_info(signed_data)
     if ac_validation_context is not None:
@@ -944,7 +1075,25 @@ async def async_validate_detached_cms(
     return StandardCMSSignatureStatus(**status_kwargs)
 
 
-async def handle_certvalidator_errors(coro):
+ResultType = TypeVar('ResultType', covariant=True)
+
+
+@dataclass(frozen=True)
+class CertvalidatorOperationResult(Generic[ResultType]):
+    """
+    Internal class to inspect error data from certvalidator.
+    """
+
+    success_result: Optional[ResultType]
+    revo_details: Optional[RevocationDetails] = None
+    error_time_horizon: Optional[datetime] = None
+    error_path: Optional[ValidationPath] = None
+    error_subindic: Optional[AdESIndeterminate] = None
+
+
+async def handle_certvalidator_errors(
+    coro: Awaitable[ResultType],
+) -> CertvalidatorOperationResult[ResultType]:
     """
     Internal error handling function that maps certvalidator errors
     to AdES status indications.
@@ -952,24 +1101,40 @@ async def handle_certvalidator_errors(coro):
     :param coro:
     :return:
     """
-    revo_details = None
+    time_horizon: Optional[datetime] = None
+    revo_details = path = None
     try:
-        return None, None, await coro
+        return CertvalidatorOperationResult(success_result=await coro)
     except InvalidCertificateError as e:
         logger.warning(e.failure_msg, exc_info=e)
         ades_status = AdESIndeterminate.CHAIN_CONSTRAINTS_FAILURE
     except TimeSlideFailure as e:
         logger.warning(e.failure_msg, exc_info=e)
         ades_status = AdESIndeterminate.NO_POE
+    except StaleRevinfoError as e:
+        logger.warning(e.failure_msg, exc_info=e)
+        # note: the way pyhanko-certvalidator handles revinfo freshness
+        # is not strictly compliant with AdES rules, but this mapping
+        # should be roughly appropriate in most cases
+        ades_status = AdESIndeterminate.TRY_LATER
+        time_horizon = e.time_cutoff
     except DisallowedAlgorithmError as e:
+        logger.warning(e.failure_msg, exc_info=e)
+        # note: this is the one from the certvalidator hierarchy, which is
+        # similar but not quite the same as the one for pyhanko itself
+        # (conceptually identical, but the contextual data is different)
+        path = e.original_path
         if e.banned_since is None:
             # permaban
             ades_status = AdESIndeterminate.CRYPTO_CONSTRAINTS_FAILURE
         else:
             # could get resolved with more POEs
             ades_status = AdESIndeterminate.CRYPTO_CONSTRAINTS_FAILURE_NO_POE
+            time_horizon = e.banned_since
     except RevokedError as e:
+        path = e.original_path
         logger.warning(e.failure_msg)
+        time_horizon = e.revocation_dt
         if e.is_side_validation:
             # don't report this as a revocation event
             ades_status = AdESIndeterminate.CERTIFICATE_CHAIN_GENERAL_FAILURE
@@ -991,7 +1156,9 @@ async def handle_certvalidator_errors(coro):
         logger.warning("Failed to build path", exc_info=e)
         ades_status = AdESIndeterminate.NO_CERTIFICATE_CHAIN_FOUND
     except ExpiredError as e:
+        path = e.original_path
         logger.warning(e.failure_msg)
+        time_horizon = e.expired_dt
         if not e.is_side_validation and e.is_ee_cert:
             # TODO modify certvalidator to perform revinfo checks on
             #  expired certs, possibly as an option. If this happens, we
@@ -1001,11 +1168,17 @@ async def handle_certvalidator_errors(coro):
         else:
             ades_status = AdESIndeterminate.CERTIFICATE_CHAIN_GENERAL_FAILURE
     except PathValidationError as e:
+        path = e.original_path
         logger.warning(e.failure_msg, exc_info=e)
-        # TODO verify whether this is appropriate
-        ades_status = AdESIndeterminate.CHAIN_CONSTRAINTS_FAILURE
+        ades_status = AdESIndeterminate.CERTIFICATE_CHAIN_GENERAL_FAILURE
     except ValidationError as e:
         logger.warning(e.failure_msg, exc_info=e)
         ades_status = AdESIndeterminate.CERTIFICATE_CHAIN_GENERAL_FAILURE
 
-    return ades_status, revo_details, None
+    return CertvalidatorOperationResult(
+        success_result=None,
+        revo_details=revo_details,
+        error_time_horizon=time_horizon,
+        error_path=path,
+        error_subindic=ades_status,
+    )
