@@ -2,7 +2,7 @@ import dataclasses
 import enum
 from dataclasses import dataclass, field
 from datetime import datetime
-from typing import List, Optional
+from typing import Generator, List, Optional
 
 from asn1crypto import x509
 from pyhanko_certvalidator.context import (
@@ -16,12 +16,21 @@ from pyhanko_certvalidator.fetchers import FetcherBackend
 from pyhanko_certvalidator.fetchers.requests_fetchers import (
     RequestsFetcherBackend,
 )
-from pyhanko_certvalidator.ltv.poe import POEManager
+from pyhanko_certvalidator.ltv.poe import (
+    KnownPOE,
+    POEManager,
+    POEType,
+    ValidationObject,
+    ValidationObjectType,
+    digest_for_poe,
+)
 from pyhanko_certvalidator.ltv.types import ValidationTimingInfo
+from pyhanko_certvalidator.policy_decl import NonRevokedStatusAssertion
 from pyhanko_certvalidator.revinfo.archival import CRLContainer, OCSPContainer
 
 from pyhanko.sign.diff_analysis import DEFAULT_DIFF_POLICY, DiffPolicy
 from pyhanko.sign.validation import KeyUsageConstraints
+from pyhanko.sign.validation.utils import CMSAlgorithmUsagePolicy
 
 __all__ = [
     'SignatureValidationSpec',
@@ -29,7 +38,7 @@ __all__ = [
     'RevinfoOnlineFetchingRule',
     'LocalKnowledge',
     'RevocationInfoGatheringSpec',
-    'KnownPOE',
+    'CMSAlgorithmUsagePolicy',
     'bootstrap_validation_data_handlers',
 ]
 
@@ -64,22 +73,56 @@ class RevocationInfoGatheringSpec:
     )
 
 
-@dataclass(frozen=True)
-class KnownPOE:
-    digest: bytes
-    poe_time: datetime
-
-
 @dataclass(frozen=True)
 class LocalKnowledge:
     known_ocsps: List[OCSPContainer] = field(default_factory=list)
     known_crls: List[CRLContainer] = field(default_factory=list)
     known_certs: List[x509.Certificate] = field(default_factory=list)
     known_poes: List[KnownPOE] = field(default_factory=list)
+    nonrevoked_assertions: List[NonRevokedStatusAssertion] = field(
+        default_factory=list
+    )
 
     def add_to_poe_manager(self, poe_manager: POEManager):
         for poe in self.known_poes:
-            poe_manager.register_by_digest(poe.digest, dt=poe.poe_time)
+            poe_manager.register_known_poe(poe)
+
+    def assert_existence_known_at(
+        self, dt: datetime
+    ) -> Generator[KnownPOE, None, None]:
+        for poe in self.known_poes:
+            yield dataclasses.replace(
+                poe,
+                poe_time=min(poe.poe_time, dt),
+                poe_type=POEType.PROVIDED,
+            )
+        for crl in self.known_crls:
+            yield KnownPOE(
+                digest=digest_for_poe(crl.crl_data.dump()),
+                poe_time=dt,
+                poe_type=POEType.PROVIDED,
+                validation_object=ValidationObject(
+                    ValidationObjectType.CRL, crl
+                ),
+            )
+        for ocsp in self.known_ocsps:
+            yield KnownPOE(
+                digest=digest_for_poe(ocsp.ocsp_response_data.dump()),
+                poe_time=dt,
+                poe_type=POEType.PROVIDED,
+                validation_object=ValidationObject(
+                    ValidationObjectType.OCSP_RESPONSE, ocsp
+                ),
+            )
+        for cert in self.known_certs:
+            yield KnownPOE(
+                digest=digest_for_poe(cert.dump()),
+                poe_time=dt,
+                poe_type=POEType.PROVIDED,
+                validation_object=ValidationObject(
+                    ValidationObjectType.CERTIFICATE, cert
+                ),
+            )
 
 
 @dataclass(frozen=True)
@@ -92,6 +135,7 @@ class SignatureValidationSpec:
     ac_validation_policy: Optional[CertValidationPolicySpec] = None
     local_knowledge: LocalKnowledge = LocalKnowledge()
     key_usage_settings: KeyUsageConstraints = KeyUsageConstraints()
+    signature_algorithm_policy: Optional[CMSAlgorithmUsagePolicy] = None
 
 
 @dataclass(frozen=True)
@@ -140,5 +184,6 @@ def bootstrap_validation_data_handlers(
         ocsps=knowledge.known_ocsps,
         certs=knowledge.known_certs,
         poe_manager=poe_manager_override,
+        nonrevoked_assertions=knowledge.nonrevoked_assertions,
     )
     return handlers
