@@ -3,10 +3,25 @@ from collections import defaultdict
 from dataclasses import dataclass
 from datetime import datetime
 from enum import unique
-from typing import ClassVar, Collection, Dict, Iterable, Optional, Set, Union
+from typing import (
+    Any,
+    ClassVar,
+    Collection,
+    Dict,
+    Iterable,
+    List,
+    Optional,
+    Set,
+    Tuple,
+    Union,
+)
 
 from asn1crypto import cms, core, crl, keys, x509
-from pyhanko_certvalidator.errors import PathBuildingError, PathValidationError
+from pyhanko_certvalidator.errors import (
+    PathBuildingError,
+    PathValidationError,
+    ValidationError,
+)
 from pyhanko_certvalidator.path import ValidationPath
 from pyhanko_certvalidator.validate import ACValidationResult
 
@@ -107,7 +122,7 @@ class SignatureStatus:
     Message digest algorithm used.
     """
 
-    validation_path: ValidationPath
+    validation_path: Optional[ValidationPath]
     """
     Validation path providing a valid chain of trust from the signer's
     certificate to a trusted root certificate.
@@ -119,6 +134,15 @@ class SignatureStatus:
     in the chain) was revoked.
     """
 
+    error_time_horizon: Optional[datetime]
+    """
+    Informational timestamp indicating a point in time where the validation
+    behaviour potentially changed (e.g. expiration, revocation, etc.).
+
+    The presence of this value by itself should not be taken as an assertion
+    that validation would have succeeded if executed before that point in time.
+    """
+
     # XXX frozenset makes more sense here, but asn1crypto doesn't allow that
     #  (probably legacy behaviour)
     key_usage: ClassVar[Set[str]] = {'non_repudiation'}
@@ -135,6 +159,11 @@ class SignatureStatus:
     See :attr:`.KeyUsageConstraints.extd_key_usage`.
     """
 
+    validation_time: Optional[datetime]
+    """
+    Reference time for validation purposes.
+    """
+
     def summary_fields(self):
         if self.trusted:
             cert_status = 'TRUSTED'
@@ -159,10 +188,15 @@ class SignatureStatus:
         Reports whether the signer's certificate is trusted w.r.t. the currently
         relevant validation context and key usage requirements.
         """
-        return self.valid and self.intact and self.trust_problem_indic is None
+        return (
+            self.valid
+            and self.intact
+            and self.trust_problem_indic is None
+            and self.validation_path is not None
+        )
 
     # TODO explain in more detail.
-    def summary(self, delimiter=','):
+    def summary(self, delimiter=',') -> str:
         """
         Provide a textual but machine-parsable summary of the validity.
         """
@@ -191,7 +225,7 @@ class SignatureStatus:
         return key_usage_settings
 
     @property
-    def _trust_anchor(self):
+    def _trust_anchor(self) -> str:
         if self.validation_path is not None:
             trust_anchor: x509.Certificate = self.validation_path[0]
             return trust_anchor.subject.human_friendly
@@ -205,7 +239,7 @@ class TimestampSignatureStatus(SignatureStatus):
     Signature status class used when validating timestamp tokens.
     """
 
-    key_usage = set()
+    key_usage: ClassVar[Set[str]] = set()
     """
     There are no (non-extended) key usage requirements for TSA certificates.
     """
@@ -291,12 +325,13 @@ class CertifiedAttributes:
         cls, results: Iterable[ACValidationResult], parse_error_fatal=False
     ):
         # first, classify the attributes and results by type
-        by_type = defaultdict(lambda: ([], []))
+        by_type: Dict[
+            str, Tuple[List[Any], List[ACValidationResult]]
+        ] = defaultdict(lambda: ([], []))
         for result in results:
-            for attr_type, attr in result.approved_attributes.items():
-                attr_type = None
+            for attr in result.approved_attributes.values():
+                attr_type = attr['type'].native
                 try:
-                    attr_type = attr['type'].native
                     values = list(attr['values'])  # force a surface-level parse
                 except ValueError as e:
                     _handle_attr_err(
@@ -320,7 +355,7 @@ class CertifiedAttributes:
             )
         return infos
 
-    def __init__(self):
+    def __init__(self: 'CertifiedAttributes'):
         self._attrs: Dict[str, CertifiedAttributeInfo] = {}
 
     def __getitem__(self, item: str) -> CertifiedAttributeInfo:
@@ -371,7 +406,7 @@ class ClaimedAttributes:
             )
         return infos
 
-    def __init__(self):
+    def __init__(self: 'ClaimedAttributes'):
         self._attrs: Dict[str, X509AttributeInfo] = {}
 
     def __getitem__(self, item: str) -> X509AttributeInfo:
@@ -413,7 +448,7 @@ class CAdESSignerAttributeAssertions:
     """
 
     ac_validation_errs: Optional[
-        Collection[Union[PathValidationError, PathBuildingError]]
+        Collection[Union[ValidationError, PathBuildingError]]
     ] = None
     """
     Attribute certificate validation errors.
@@ -565,7 +600,7 @@ class StandardCMSSignatureStatus(SignerAttributeStatus, SignatureStatus):
         sections.append(("Bottom line", bottom_line))
         return '\n'.join(fmt_section(hdr, body) for hdr, body in sections)
 
-    def pretty_print_sections(self):
+    def pretty_print_sections(self) -> List[Tuple[str, str]]:
         cert: x509.Certificate = self.signing_cert
 
         # TODO add section about ACs
@@ -617,8 +652,8 @@ class StandardCMSSignatureStatus(SignerAttributeStatus, SignatureStatus):
                 f"The token is guaranteed to be newer than the signature.\n"
                 f"{tst_status.describe_timestamp_trust()}"
             )
-        content_tst_status = self.timestamp_validity
-        if tst_status is not None:
+        content_tst_status = self.content_timestamp_validity
+        if content_tst_status is not None:
             ts = content_tst_status.timestamp
             timing_infos.append(
                 f"Content timestamp token: {ts.isoformat()}\n"
@@ -676,7 +711,7 @@ class SignatureCoverageLevel(OrderedEnum):
 
 @dataclass(frozen=True)
 class ModificationInfo:
-    coverage: SignatureCoverageLevel = None
+    coverage: Optional[SignatureCoverageLevel] = None
     """
     Indicates how much of the document is covered by the signature.
     """
@@ -693,6 +728,15 @@ class ModificationInfo:
       by the difference policy will be stored in this attribute.
     """
 
+    docmdp_ok: Optional[bool] = None
+    """
+    Indicates whether the signature's
+    :attr:`~.ModificationInfo.modification_level` is in line with the document
+    signature policy in force.
+
+    If ``None``, compliance could not be determined.
+    """
+
     @property
     def modification_level(self) -> Optional[ModificationLevel]:
         """
@@ -723,15 +767,6 @@ class ModificationInfo:
 class PdfSignatureStatus(ModificationInfo, StandardCMSSignatureStatus):
     """Class to indicate the validation status of a PDF signature."""
 
-    docmdp_ok: Optional[bool] = None
-    """
-    Indicates whether the signature's
-    :attr:`~.ModificationInfo.modification_level` is in line with the document
-    signature policy in force.
-
-    If ``None``, compliance could not be determined.
-    """
-
     has_seed_values: bool = False
     """
     Records whether the signature form field has seed values.
